#include "MyMultiLineEdit.moc"
#include <qpopupmenu.h>
#include <iostream>

MyMultiLineEdit::MyMultiLineEdit(DVertSplit * parent = 0, const char * name = 0, MyMainWindow* grandpa=0, bool notCentral = 0):QMultiLineEdit(parent,name)
{
  //palette = QMultiLineEdit::palette();
  if (parent)
    parent->setNormalPalette();
  currSL=0;
  currSC=0;
  currEL=0;
  currEC=0;
  parentWid=parent;
  grandpaWid = grandpa;
  currKN = 0;
  currWorkingKN = 0;
  _notCentral = notCentral;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: findKN
// Parameters:	(int, int)
//
// Finds the index in the currLCArray that contains line and col. If none
// exists, return -1.
//         								    
//////////////////////////////////////////////////////////////////////////////

int MyMultiLineEdit::findKN(int line, int col) {
  int i = 0;
  int sL, sC, eL, eC;
  for (int i = 0; i < currNumStmts; i++) {
    sL = currLCArray[i].startLine;
    sC = currLCArray[i].startCol;
    eL = currLCArray[i].endLine;
    eC = currLCArray[i].endCol;
    if ((sL <= line) && (eL >= line) && (sC <= col) && (eC >= col))
      return i;
  }
  return -1;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getCurrKN
// Parameters:	(void)
//
// Finds the kn of the currently highlight text. If no text is highlighted,
// returns the kn of the current rectangle.
//         								    
//////////////////////////////////////////////////////////////////////////////

int MyMultiLineEdit::getCurrKN() {
  if (markedText())
    return currWorkingKN;
  else
    return currKN;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: highlightKN
// Parameters:	(int)
//
// Causes the range indicated by the index in the currLCArray to be
// highlighted.
//         								    
//////////////////////////////////////////////////////////////////////////////

void MyMultiLineEdit::highlightKN(int kn) {
  int sL, sC, eL, eC;
  sL = currLCArray[kn].startLine;
  sC = currLCArray[kn].startCol;
  eL = currLCArray[kn].endLine;
  eC = currLCArray[kn].endCol;
  setSelection(sL,sC,eL,eC);
  currWorkingKN = kn;
}


void MyMultiLineEdit::updateBCA(int* cBA, int* cCA, int size)
{
  currBA=cBA;
  currCA=cCA;
  currArraySize=size;
}

void MyMultiLineEdit::getInfo(int kn, int* sl, int* sc, int* el, int* ec)
{
  *sl=currLCArray[kn].startLine;
  *sc=currLCArray[kn].startCol;
  *el=currLCArray[kn].endLine;
  *ec=currLCArray[kn].endCol;
}

void MyMultiLineEdit::setCurr(int kn, LCSubstring* currLCA, int numStmts)
{
  currSL=currLCA[kn-1].startLine;
  currSC=currLCA[kn-1].startCol;
  currEL=currLCA[kn-1].endLine;
  currEC=currLCA[kn-1].endCol;
  currNumStmts = numStmts;
  currLCArray = currLCA;
  currKN = kn-1;
}

void MyMultiLineEdit::markStatement(QColor color, QPainter *p,int sc, int ec, int row)
{
  QString currLine = textLine(row);
  int w1 = p->fontMetrics().width(currLine.left(sc)) + 1;
  int w2 = p->fontMetrics().width(currLine.mid(sc, ec-sc)) + 1;
  int y = cellHeight(row)-1;
  p->setPen(color);
  p->drawLine(w1,0,w1,y);
  p->drawLine(w1,0,w2+w1,0);
  p->drawLine(w2+w1,0,w2+w1,y);
  p->drawLine(w1,y,w2+w1,y);
  p->setPen(black);
  //p->drawRect(r);
}

void MyMultiLineEdit::paintCell(QPainter* p, int row, int col)
{
  QMultiLineEdit::paintCell(p,row,col);
  int i = 0;
  while (i < currArraySize) {
    if (currBA[i])
      {
	int sl,sc,el,ec;
	getInfo(i,&sl,&sc,&el,&ec);
	if (!((row < sl) || (row > el)))
	  markStatement(red,p,sc,ec,row);
      }
    i++;
  }
  i=0;
  while (i < currArraySize) {
    if (currCA[i])
      {
	int sl,sc,el,ec;
	getInfo(i,&sl,&sc,&el,&ec);
	if (!((row < sl) || (row > el)))
	  markStatement(blue,p,sc,ec,row);
      }
    i++;
  }
  if (!_notCentral)
    if (!((row < currSL) || (row > currEL)))
      markStatement(darkGreen,p,currSC,currEC,row);
	
  
  //if ((row < currSL) || (row > currEL)) {
  //  QMultiLineEdit::paintCell(p,row,col);
  //}
  //else {
  //  QMultiLineEdit::paintCell(p,row,col);
  //  markStatement(darkGreen,p,currSC,currEC,row);
  //}
  
}

void MyMultiLineEdit::mousePressEvent(QMouseEvent * e)
{
  if ( e->button() == RightButton ) {
    int x;
    int y;
    QMouseEvent e_new(QEvent::MouseButtonPress, e->pos(),LeftButton,LeftButton);
    QMultiLineEdit::mousePressEvent(&e_new);
    //QMultiLineEdit::mouseReleaseEvent(&e_new);
    getCursorPosition(&x, &y);
    int kn = findKN(x,y);
    grandpaWid->enableDisable(kn);
    //cout << "kn:"<<kn<<endl;
    if (kn!=-1) {
      highlightKN(kn);
      QPopupMenu *popup = new QPopupMenu( this );
      int id[3];
      //cout << (id[0] = popup->insertItem("tesst")) << endl;
      if (_notCentral) {
	id[0] = popup->insertItem("break",this,SLOT(doBreak()));
	id[1] = popup->insertItem("condition",this,SLOT(doCondition()));
	id[2] = popup->insertItem("clear",this,SLOT(doClear()));
      }
      else {
	id[0] = popup->insertItem("break",grandpaWid,SLOT(doBreak()));
	id[1] = popup->insertItem("condition",grandpaWid,SLOT(doCondition()));
	id[2] = popup->insertItem("clear",grandpaWid,SLOT(doClear()));
      }
      if ((currBA[kn]) || (currCA[kn])) {
	popup->setItemEnabled(id[0],false);
	popup->setItemEnabled(id[1],false);
	popup->setItemEnabled(id[2],true);
      }
      else {
	popup->setItemEnabled(id[0],true);
	popup->setItemEnabled(id[1],true);
	popup->setItemEnabled(id[2],false);
      }
      int r = popup->exec( e->globalPos() );
      grandpaWid->enableDisable(kn);
      //if ( r == id[0] ) {
	
      //}
    }
    else {
      currWorkingKN = currKN;
      setCursorPosition (x,y,0);
      //the following line will remain commented out unless the new version
      //of qt is installed where QMultiLineEdit inherits QScrollView (through
      //QTextEdit) instead of QTableView as it does now.
      //center(x,y);
    }
    return;
  }
  else
    QMultiLineEdit::mousePressEvent(e);
  
}


void MyMultiLineEdit::mouseReleaseEvent(QMouseEvent * e)
{
  QMultiLineEdit::mouseReleaseEvent(e);
  int x = 0;
  int y = 0;
  getCursorPosition(&x, &y);
  int kn = findKN(x,y);
  grandpaWid->enableDisable(kn);
  if (kn!=-1)
    highlightKN(kn);
  else {
    currWorkingKN = currKN;
    setCursorPosition (x,y,0);
  }
}

bool MyMultiLineEdit::close( bool alsoDelete ) {
  //figure out the name of the transformation by looking at the first line
  //of the contents at taking the 16th letter to the end
  int transNameLength = lineLength(0) - 15;
  char* transName = new char[transNameLength + 1];
  const char* line0 = textLine(0); //get the current textLine, but subtract the word TRANSFORMATION to get the name of the transformation
  for (int i = 0; i<transNameLength; i++)
    transName[i] = line0[i+15];
  transName[transNameLength] = '\0';
  grandpaWid->removeFromTransList(transName);
  delete [] transName;
  return QWidget::close(alsoDelete);
}

//SLOTS

void MyMultiLineEdit::doBreak()
{
  grandpaWid->clearParsedArray();
  char** parsedArray = grandpaWid->getParsedArray();

  // The following code puts into parsedArray the command:
  // <name-of-transformation> break key_number, for example:
  // <MyTestCNF> break 4
  // The name of the transformation is gotten from the first line of the
  // contents (which is TRANSFORMATION name-of-transformation). So if that
  // formatting change, then the following code will not be correct.

  int transNameLength = lineLength(0) - 15;
  //parsedArray[0] should contain <name-of-transformation>
  parsedArray[0] = new char[strlen("<>") + transNameLength + 1];
  char* transName = new char[transNameLength + 1];
  const char* line0 = textLine(0); //get the current textLine, but subtract the word TRANSFORMATION to get the name of the transformation
  for (int i = 0; i<transNameLength; i++)
    transName[i] = line0[i+15];
  transName[transNameLength] = '\0';
  sprintf(parsedArray[0], "<%s>", transName);
  delete [] transName;

  //parsedArray[1] should contain the word: break
  parsedArray[1] = new char[strlen("break") + 1];
  sprintf(parsedArray[1], "break");

  //parsedArray[2] should contain the string version of the key_number
  int kn = getCurrKN() + 1;
  char* temp = new char[12];
  for (int i = 0; i < 12; i++)
    temp[i] = '\0';
  sprintf(temp, "%d", kn);
  int count = 0;
  for (int i = 0; i < 12; i++)
    if (temp[i] != '\0')
      count++;
  parsedArray[2] = new char[count+1];
  for (int i = 0; i < count; i++)
    parsedArray[2][i] = temp[i];

  //add contents of parsedArray to the console
  parsedArray[2][count] = '\0';
  grandpaWid->insertConsoleText(parsedArray[0]);
  grandpaWid->insertConsoleText(" ");
  grandpaWid->insertConsoleText(parsedArray[1]);
  grandpaWid->insertConsoleText(" ");
  grandpaWid->insertConsoleText(parsedArray[2]);
  grandpaWid->insertConsoleText("\n");

  //return control to the debugger
  grandpaWid->dbGo();
}

void MyMultiLineEdit::doCondition()
{
  grandpaWid->clearParsedArray();
  char** parsedArray = grandpaWid->getParsedArray();

  // The following code puts into parsedArray the command:
  // <name-of-transformation> condition key_number, for example:
  // <MyTestCNF> condition 4
  // The name of the transformation is gotten from the first line of the
  // contents (which is TRANSFORMATION name-of-transformation). So if that
  // formatting change, then the following code will not be correct.

  int transNameLength = lineLength(0) - 15;
  //parsedArray[0] should contain <name-of-transformation>
  parsedArray[0] = new char[strlen("<>") + transNameLength + 1];
  char* transName = new char[transNameLength + 1];
  const char* line0 = textLine(0); //get the current textLine, but subtract the word TRANSFORMATION to get the name of the transformation
  for (int i = 0; i<transNameLength; i++)
    transName[i] = line0[i+15];
  transName[transNameLength] = '\0';
  sprintf(parsedArray[0], "<%s>", transName);
  delete [] transName;

  //parsedArray[1] should contain the word: break
  parsedArray[1] = new char[strlen("condition") + 1];
  sprintf(parsedArray[1], "condition");

  //parsedArray[2] should contain the string version of the key_number
  int kn = getCurrKN() + 1;
  char* temp = new char[12];
  for (int i = 0; i < 12; i++)
    temp[i] = '\0';
  sprintf(temp, "%d", kn);
  int count = 0;
  for (int i = 0; i < 12; i++)
    if (temp[i] != '\0')
      count++;
  parsedArray[2] = new char[count+1];
  for (int i = 0; i < count; i++)
    parsedArray[2][i] = temp[i];

  //add contents of parsedArray to the console
  parsedArray[2][count] = '\0';
  grandpaWid->insertConsoleText(parsedArray[0]);
  grandpaWid->insertConsoleText(" ");
  grandpaWid->insertConsoleText(parsedArray[1]);
  grandpaWid->insertConsoleText(" ");
  grandpaWid->insertConsoleText(parsedArray[2]);
  grandpaWid->insertConsoleText("\n");

  //return control to the debugger
  grandpaWid->dbGo();
}

void MyMultiLineEdit::doClear()
{
  grandpaWid->clearParsedArray();
  char** parsedArray = grandpaWid->getParsedArray();

  // The following code puts into parsedArray the command:
  // <name-of-transformation> clear key_number, for example:
  // <MyTestCNF> clear 4
  // The name of the transformation is gotten from the first line of the
  // contents (which is TRANSFORMATION name-of-transformation). So if that
  // formatting change, then the following code will not be correct.

  int transNameLength = lineLength(0) - 15;
  //parsedArray[0] should contain <name-of-transformation>
  parsedArray[0] = new char[strlen("<>") + transNameLength + 1];
  char* transName = new char[transNameLength + 1];
  const char* line0 = textLine(0); //get the current textLine, but subtract the word TRANSFORMATION to get the name of the transformation
  for (int i = 0; i<transNameLength; i++)
    transName[i] = line0[i+15];
  transName[transNameLength] = '\0';
  sprintf(parsedArray[0], "<%s>", transName);
  delete [] transName;

  //parsedArray[1] should contain the word: break
  parsedArray[1] = new char[strlen("clear") + 1];
  sprintf(parsedArray[1], "clear");

  //parsedArray[2] should contain the string version of the key_number
  int kn = getCurrKN() + 1;
  char* temp = new char[12];
  for (int i = 0; i < 12; i++)
    temp[i] = '\0';
  sprintf(temp, "%d", kn);
  int count = 0;
  for (int i = 0; i < 12; i++)
    if (temp[i] != '\0')
      count++;
  parsedArray[2] = new char[count+1];
  for (int i = 0; i < count; i++)
    parsedArray[2][i] = temp[i];

  //add contents of parsedArray to the console
  parsedArray[2][count] = '\0';
  grandpaWid->insertConsoleText(parsedArray[0]);
  grandpaWid->insertConsoleText(" ");
  grandpaWid->insertConsoleText(parsedArray[1]);
  grandpaWid->insertConsoleText(" ");
  grandpaWid->insertConsoleText(parsedArray[2]);
  grandpaWid->insertConsoleText("\n");

  //return control to the debugger
  grandpaWid->dbGo();
}

