//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Daniel Abadi 
//  FILE: SymbolList.C
//  PROJ: COKO DEBUGGER
//  DATE: SUMMER 2001                                                      
//
// SYNOPSIS
//  Classes for Symbol List.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _SYMBOLLIST_C_
#define _SYMBOLLIST_C_

#include "SymbolList.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: SymbolNode
// Parameters:	(char*, int)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

SymbolNode::SymbolNode(const char * a, int num):	
  _next(NULL),
  _value(NULL)
{
  _num = num;
  _value = new char[strlen(a) + 1];
  strcpy(_value, a);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~SymbolNode
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////
  
SymbolNode::~SymbolNode(void) 
{
  if (_value) { delete _value; _value = NULL; }
  _next = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: display
// Parameters:	(void)
//
// For debugging purposes only
//         								    
//////////////////////////////////////////////////////////////////////////////
  
void SymbolNode::display(void) 
{
  cout << _num << ": ";
  if (_value)
    cout <<  _value  << endl;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insert
// Parameters: (char*, int)	
//
// Insert a new node with _value = a and _num = num to the list.
//         								    
//////////////////////////////////////////////////////////////////////////////

void SymbolList::insert(const char* a, int num) 
{
  if (!_head)
	_head = _tail = new SymbolNode(a, num); 
  else 
	{
	  SymbolNode * tmp = new SymbolNode(a, num);
	  _tail->_next = tmp;
	  _tail = tmp;
	  tmp = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: pop
// Parameters: (void)
//
// Pop off a front node from the list.
// Return success value of the pop.
//         								    
//////////////////////////////////////////////////////////////////////////////

int SymbolList::pop(void)
{
  if (!_head)
	return 0;

  SymbolNode * tmpnode = _head;

  if (_head == _tail)
	_head = _tail = NULL;
  else
	_head = _head->_next;

  if (tmpnode->_value) 
	{ delete tmpnode; tmpnode = NULL; }
  else
	{
	  cout << "Error @ SymbolList : pop " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return 0;
	}

  return 1;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: top
// Parameters: (void)
//
// Return _value of the top node.
//         								    
//////////////////////////////////////////////////////////////////////////////

char* SymbolList::top(void)
{
  if (!_head)
	return NULL;

  if (_head->_value) 
	return _head->_value;
  else
	{
	  cout << "Error @ SymbolList : top " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: empty
// Parameters: (void)
//
// Is the list empty?
//         								    
//////////////////////////////////////////////////////////////////////////////

int SymbolList::empty(void) 
{ 
  return (!_head); 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: initIterator 
// Parameters: (void)
//
// Initialize _iterator.
// This method is used with getValue method.
//         								    
//////////////////////////////////////////////////////////////////////////////

void SymbolList::initIterator(void)
{
  _iterator = _head;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorValue
// Parameters:	(void)
//
// Returns _value of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

char* SymbolList::iteratorValue(void)
{
  if (_iterator)
	return _iterator->_value;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorNextValue
// Parameters:	(void)
//
// Returns _value of the next node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

char* SymbolList::iteratorNextValue(void)
{
  if ((_iterator) && (_iterator->_next))
	return _iterator->_next->_value;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorNum
// Parameters:	(void)
//
// Returns _num of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int SymbolList::iteratorNum(void)
{
  if (_iterator)
	return _iterator->_num;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorNextNum
// Parameters:	(void)
//
// Returns _num of the next node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int SymbolList::iteratorNextNum(void)
{
  if ((_iterator) && (_iterator->_next))
	return _iterator->_next->_num;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: setIteratorNum
// Parameters:	(int)
//
// Set _num of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

void SymbolList::setIteratorNum(int x)
{
  if (_iterator)
	_iterator->_num = x;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: setIteratorNextNum
// Parameters:	(int)
//
// Set _num of the next node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

void SymbolList::setIteratorNextNum(int x)
{
  if ((_iterator) && (_iterator->_next))
	_iterator->_next->_num = x;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: deleteNext
// Parameters:	(void)
//
// Deletes the next node that the _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int SymbolList::deleteNext(void)
{
  if ((_iterator) && (_iterator->_next))
    {
      SymbolNode * tmpnode = _iterator->_next;
      _iterator->_next=tmpnode->_next;
      if (!(tmpnode->_next))
	_tail = _iterator;
      if (tmpnode->_value) 
	{ delete tmpnode; tmpnode = NULL; }
      else
	{
	  cout << "Error @ SymbolList : deleteNext " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return 0;
	}
    }
  else
    return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getNextValue
// Parameters:	(void)
//
// Point _iterator to the next node and returns the _value of the next node
// after that.
// One can call this methods repeatly to get the _value of all the nodes.
// However, initIterator has to be called before the initial call to this 
// method.  
//         								    
//////////////////////////////////////////////////////////////////////////////

char* SymbolList::getNextValue()
{
  char* return_val;
  if ((_iterator) && (_iterator->_next) && (_iterator->_next->_next))
	{
	   _iterator = _iterator->_next;
	  return_val = _iterator->_next->_value;
	}
  else
	return_val = NULL;
  return return_val;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: find
// Parameters:	(const char*)
//
// Points _iterator to the previous node from the node that has the given
// value, and returns 1 if that value was found.
// If the node that has the given value is the _head of the list, the it
// points _iterator to the head and returns 2.
// If the given value is not in the list, _iterator points to NULL and
// 0 is returned.
//         								    
//////////////////////////////////////////////////////////////////////////////

int SymbolList::find(const char* value)
{
  initIterator();
  char* temp;
  if (_iterator)
    if (strcmp(value,iteratorValue()) == 0)
      return 2;
    else {
      if (_iterator->_next) {
	if (strcmp(value,iteratorNextValue()) == 0)
	  return 1;
	else {
	  while (temp = getNextValue())
	    if (strcmp(value,temp) == 0)
	      return 1;
	  return 0;
	}
      }
      else 
	return 0;
    }
  else
    return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: SymbolList
// Parameters: (void)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

SymbolList::SymbolList():
  _head(NULL),
  _tail(NULL),
  _iterator(NULL)
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:~SymbolList
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

SymbolList::~SymbolList()
{
  SymbolNode * tmp = _head;
  _tail = NULL;
  while (_head)
   	{
 	  _head = _head->_next;
 	  delete tmp;
 	  tmp = _head;
 	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: display
// Parameters:	(void)
//
// For debugging purposes only. Displays the contents of the entire list.
//         								    
//////////////////////////////////////////////////////////////////////////////
void SymbolList::display()
{
  cout << "DISPLAYING LIST" << endl;
  initIterator();
  if (_iterator)
    _iterator->display();
  //if (_iterator->_next)
  //  _iterator->_next->display();
  
  for(;;)
    if (getNextValue())
      _iterator->display();
    else
      break;
  
  if ((_iterator) && (_iterator->_next))
    _iterator->_next->display();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: delete_marked_nodes
// Parameters:	(void)
//
// delete all nodes with _num of -1 in list
//         								    
//////////////////////////////////////////////////////////////////////////////

void SymbolList::delete_marked_nodes(void)
{
  initIterator();
  while ((_iterator) && (_iterator->_num == -1)) {
    pop();
    initIterator();
  }
  if (_iterator) {
    while (_iterator->_next) {
      if (iteratorNextNum() == -1)
	deleteNext();
      else
	_iterator = _iterator->_next;
    }
  }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: delete_all_nodes
// Parameters:	(void)
//
// delete all nodes in list
//         								    
//////////////////////////////////////////////////////////////////////////////

void SymbolList::delete_all_nodes(void)
{
  initIterator();
  while (_iterator) {
    pop();
    initIterator();
  }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: findCorrespondingSym
// Parameters:	(int)
//
// Returns the symbol with corresponding to the integer input x
// If the list was maintained correctly, there should be only one such symbol
// Otherwise, the first such symbol is returned
//         								    
//////////////////////////////////////////////////////////////////////////////

char* SymbolList::findCorrespondingSym(int x)
{
  initIterator();
  while (_iterator) {
    if (iteratorNum() == x)
      return iteratorValue();
    else
    _iterator = _iterator->_next;
  }
}

#endif

