//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Daniel Abadi 
//  FILE: UsesList.C
//  PROJ: COKO DEBUGGER
//  DATE: SUMMER 2001                                                      
//
// SYNOPSIS
//  Classes for Uses List.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _USESLIST_C_
#define _USESLIST_C_

#include "UsesList.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: UsesNode
// Parameters:	(CRule*)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

UsesNode::UsesNode(CRule * a):	
  _next(NULL),
  _rule(NULL),
  _ruleblock(NULL)
{
  _rule = a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: UsesNode
// Parameters:	(CRuleBlock*)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

UsesNode::UsesNode(CRuleBlock * a):	
  _next(NULL),
  _rule(NULL),
  _ruleblock(NULL)
{
  _ruleblock = a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~UsesNode
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////
  
UsesNode::~UsesNode(void) 
{
  if (_rule) { _rule = NULL; }
  if (_ruleblock) { _ruleblock = NULL; }
  _next = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insert
// Parameters: (CRule*)	
//
// Insert a new node with _rule = a to the list.
//         								    
//////////////////////////////////////////////////////////////////////////////

void UsesList::insert(CRule* a) 
{
  if (!_head)
	_head = _tail = new UsesNode(a); 
  else 
	{
	  UsesNode * tmp = new UsesNode(a);
	  _tail->_next = tmp;
	  _tail = tmp;
	  tmp = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insert
// Parameters: (CRuleBlock*)	
//
// Insert a new node with _ruleblock = a to the list.
//         								    
//////////////////////////////////////////////////////////////////////////////

void UsesList::insert(CRuleBlock* a) 
{
  if (!_head)
	_head = _tail = new UsesNode(a); 
  else 
	{
	  UsesNode * tmp = new UsesNode(a);
	  _tail->_next = tmp;
	  _tail = tmp;
	  tmp = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: pop
// Parameters: (void)
//
// Pop off a front node from the list.
// Return success value of the pop.
//         								    
//////////////////////////////////////////////////////////////////////////////

int UsesList::pop(void)
{
  if (!_head)
	return 0;

  UsesNode * tmpnode = _head;

  if (_head == _tail)
	_head = _tail = NULL;
  else
	_head = _head->_next;

  if ((tmpnode->_rule) || (tmpnode->_ruleblock))
	{ delete tmpnode; tmpnode = NULL; }
  else
	{
	  cout << "Error @ UsesList : pop " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return 0;
	}

  return 1;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: topRule
// Parameters: ()
//
// Return _rule of the top node.
//         								    
//////////////////////////////////////////////////////////////////////////////

CRule* UsesList::topRule(void)
{
  if (!_head)
    return NULL;
  
  return _head->_rule;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: topRuleBlock
// Parameters: ()
//
// Return _ruleblock of the top node.
//         								    
//////////////////////////////////////////////////////////////////////////////

CRuleBlock* UsesList::topRuleBlock(void)
{
  if (!_head)
    return NULL;
  
  return _head->_ruleblock;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: empty
// Parameters: (void)
//
// Is the list empty?
//         								    
//////////////////////////////////////////////////////////////////////////////

int UsesList::empty(void) 
{ 
  return (!_head); 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: initIterator 
// Parameters: (void)
//
// Initialize _iterator.
// This method is used with getValue method.
//         								    
//////////////////////////////////////////////////////////////////////////////

void UsesList::initIterator(void)
{
  _iterator = _head;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorRule
// Parameters:	(void)
//
// Returns _rule of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

CRule* UsesList::iteratorRule(void)
{
  if (_iterator)
	return _iterator->_rule;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorRuleBlock
// Parameters:	(void)
//
// Returns _ruleblock of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

CRuleBlock* UsesList::iteratorRuleBlock(void)
{
  if (_iterator)
	return _iterator->_ruleblock;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getValue
// Parameters:	(void)
//
// Returns next _rule of the node that _iterator is pointing to.
// After a call to this method, _iterator will point to the next node.
// One can call this methods repeatly to get the _value of all the nodes.
// However, initIterator has to be called before the initial call to this 
// method.  
//         								    
//////////////////////////////////////////////////////////////////////////////

CRule* UsesList::getValue()
{
  CRule* return_val;
  if (_iterator)
	{
	  return_val = _iterator->_rule;
	  _iterator = _iterator->_next;
	}
  else
	return_val = NULL;
  return return_val;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: UsesList
// Parameters: (void)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

UsesList::UsesList():
  _head(NULL),
  _tail(NULL),
  _iterator(NULL)
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:~UsesList
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

UsesList::~UsesList()
{
  UsesNode * tmp = _head;
  _tail = NULL;
  while (_head)
   	{
 	  _head = _head->_next;
 	  delete tmp;
 	  tmp = _head;
 	}
}

#endif

