#ifndef _VC_H_
#define _VC_H_
#include <stdio.h>
#include <qapplication.h>
#include <qmainwindow.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include "DEYesNoDialogue.H"
#include "MyMultiLineEdit.H"
#include "DEPaintWindow.H"
#include "DEGroupBox.H"
#include "DEVisConsole.H"
#include "DESymbolVisList.H"
#include <qpixmap.h>
#include <qcolor.h>
#include <qpainter.h>
#include "DEStackWindow.H"
#include "DETransformationList.H"
#include "DEScrollWidget.H"

#include "DEDebugController.H"

class MyMultiLineEdit;
class DVisConsole;
class DVertSplit;
class DHorizSplit;
class DPaintWindow;
class DStackWindow;
class DTransformationList;

class MyMainWindow : public QMainWindow
{
  Q_OBJECT
public:
  MyMainWindow(/*NonType*, CRuleBlock*, */DDebugController*, RuleBlockList*, RuleBlockList*, DTransformationList*);
  // constructor
  void displayRBInfo(char*, int, LCSubstring *, int, NonType*, NonType*, SymbolList*, char*, CState*);
  // This is the central method of this class. It is called from the breakLoop
  // method of the debug controller after it has figured out the values of all
  // of the above parameters. The first 3 parameters are used to display the
  // text of the current transformation in main window, the next 4 parameters
  // are used to display the root and current KOLA trees in their respective
  // windows. The final parameter is the current state of the system and is
  // used by the setCurrInformation method to find the values of variables.
  // After passing all of this information to their respective windows, these
  // windows are refreshed.
  char** getParsedArray();
  // This method returns the parsedArray variable. It is called from the debug
  // controller when it is ready to analyze the input.
  bool close(bool);
  // Calls the quit() slot.
  void insertConsoleText(const char*);
  // This method takes some text (toInsert) and inserts it at the end of the
  // console window.
  void dealWithSelection(QString*);
  // This method is called when the user typed in a command. This command is
  // passed through the QString* parameter str. This string is then parsed by
  // the debugger and control is returned to the Debug Controller.
  void clearParsedArray();
  // This method deletes every element in the member variable parsedArray and
  // frees its corresponding memory.
  void setCurrInformation(SymbolList*, char*, NonType*, CState* s);
  // This method updates the important variables in every paint window that
  // displays the KOLA trees and the central widget (textual transformation
  // display) window. It deletes windows that are no longer useful and
  // refreshes the ones that are.
  void enableDisable(int);
  // This method makes sure that both the break/condition or clear options
  // aren't available at the same time. If the current key number has any kind
  // of break on it, then the break and condition buttons and commands (in the
  // command menu) are disabled while the clear button and command is enabled.
  // If the key number has not break, then the reverse is done.
  int checkDoAgain(int);
  // This method takes an integer parameter (which) that specifies which of the
  // three dialog boxes to display. The corrosponding dialog box is then
  // the displyed, and the user response is returned.
  int getDisplayRT();
  // This method checks to see if the root KOLA tree display window is
  // currently being displayed. If so, then it returns true, otherwise false.
  int getDisplayCT();
  // This method checks to see if the current KOLA tree display window is
  // currently being displayed. If so, then it returns true, otherwise false.
  void setDisplayRT(int);
  // This method takes a variable int (x) and will display the root KOLA tree
  // display window to true if x == 0. Otherwise, it will hide this window. It
  // updates the view menu accordingly.
  void setDisplayCT(int);
  // This method takes a variable int (x) and will display the current KOLA
  // tree display window to true if x == 0. Otherwise, it will hide this
  // window. It updates the view menu accordingly.
  void setSVL(DSymbolVisList* svl);
  // This method sets the member variable _svl to be equal to the symbol visual
  // display list parameter (svl)
  QPopupMenu* getView();
  // This method returns the view popupmenu stored in member variable: view
  int insertIntoView(char*);
  // This method inserts into the view menu an new item labeled by the
  // parameter: toInsert. It then connects this item SIGNAL to the SLOT:
  // showItem. Finally, it returns the index of this new inserted item in the
  // view menu.
  void removeFromView(int);
  // This method removes from the view menu the item with the index
  // corresponding to the int parameter: index.
  NonType* doIsolate(NonType*,DWhichList*);
  // This method refreshes the KOLA tree isolation starting from a particular
  // root tree (rootTree) using a particular Which List (wl). Each element
  // in wl is an integer indicating how to travese the root tree from the
  // given point by telling it which child to follow. For instance, if the
  // which list contains: 2 1 1, then this means that the isolation is the
  // second child of the first child of the first child of the root tree. This
  // method returns the tree derived from this isolation algorithm.
  void insertIntoSVL(char*,DWhichList*,int,DScrollWidget*,DPaintWindow*,int);
  // This method simply passes along these parameters to the insert method of
  // the symbol visual display list. This list contains the windows of all of
  // the additional windows besides the root KOLA tree and current KOLA tree
  // display windows (hence isolations and variable display windows).
  void displayStack();
  // This method recalculates what should be in the stack window by calling the
  // displayStack method on the _curr_rbList and then sets the text of the
  // stack display window to be this returned text. The stack window is then
  // display and the view menu is updated accordingly.
  void dbGo();
  // This method simply tells the debug controller that it is ok to regain
  // control by calling its setReadyToGo method.
  void removeFromTransList(char*);
  // If the user closed a transformation window, we assume that the user no
  // longer is interested in it, so it gets removed from the list of viewable
  // transformations.
  void expandTrans(char* transName = NULL);
  // This method will expand a transformation (meaning it will open a new
  // window with the transformation code). Which transformation is expanded can
  // be indicated in two ways. The first way is by simply passing the name of
  // the transformation to the method (parameter transName). The other way is
  // by double clicking the appropriate line in the stack window which this
  // method can figure out by seeing which line is active in the stack display
  // window.
  void updatePaintWindows(NonType*, NonType*, SymbolList*, char*);
  // This takes a pointer to the root KOLA tree (rtTree) and current KOLA tree
  // (ctTree) and sets these to be the trees of the root display and current
  // display windows. In addition, it passes the SymbolList (sym) and execName
  // string to these windows. Finally it repaints them.
  void closeAllSVLWindows();
  // This method goes though each element in the symbol visual display list
  // (_svl) and closes and deletes each window.

public slots:
  void quit();
  // This slot gets called if the user tried to exit the program (by pressing
  // the exit button, by closing the window, etc). This method places the word
  // quit on the console and returns control to the debug controller which
  // will make sure that the user really wants to quit.
  void step();
  // This slot gets called if the step button is pressed. The word "step" is
  // written to the console and control is given back to the debug controller.
  void next();
  // This slot gets called if the next button is pressed. The word "next" is
  // written to the console and control is given back to the debug controller.
  void cont();
  // This slot gets called if the continue button is pressed. The word
  // "continue" is written to the console and control is given back to the
  // debug controller.
  void doBreak(); //breaking naming convention because break name not allowed
  // This slot gets called if the break button was pressed. It figures out
  // which key number the user wanted the place the break upon, and enters
  // break <kn> into the console, updates the parsedArray, and returns control
  // to the Debug Controller.
  void doCondition();
  // This slot gets called if the condition button was pressed. It figures out
  // which key number the user wanted the place the conditional clear break
  // upon, and enters condition <kn> into the console, updates the parsedArray,
  // and returns control to the Debug Controller.
  void doClear();
  // This slot gets called if the clear button was pressed. It figures out
  // which key number the user wanted the clear the break upon, and enters
  // clear <kn> into the console, updates the parsedArray, and returns control
  // to the Debug Controller.
  void doRun();
  // This slot gets called if the run button was pressed. It places the word
  // "run" on the console and returns control to the debug controller which
  // which make sure that the user really wanted to restart execution of the
  // root transformation.
  void toggleStack();
  // This slot gets called if the stack option in the view menu is selected.
  // If the stack window was currently being displayed, it gets hidden.
  // Otherwise it gets displayed.
  void doStack();
  // This slot gets called if the stack window is to be refreshed or displayed.
  // It places the word "stack" on the console and returns control to the debug
  // controller.
  void showRT();
  // This slot will display the root KOLA tree window if it is currently
  // hidden, and will hide it if it is currently being displayed. The view
  // menu is updated accordingly.
  void showCT();
  // This slot will display the current KOLA tree window if it is currently
  // hidden, and will hide it if it is currently being displayed. The view
  // menu is updated accordingly.
  void showItem(int);
  // This slot takes an int (index) which is the index of an item in the view
  // menu and will display that item if it is currently hidden, and will hide
  // it if it is currently being displayed. The view menu is updated
  // accordingly.
  void showTransformation(int);
  // This slot takes an int (index) which is the index of an item (in
  // particular a transformation) in the view menu and will display that
  // transformation if it is currently hidden, and will hide it if it is
  // currently being displayed. The view menu is updated accordingly.
  void doExpandTrans();
  // This slot is called from the popup menu derived from right-clicking on
  // the stack window. It will put the words "track <name-of-trans> to the
  // console and allow the debugger to execute the command.

private:
  QPopupMenu *file;
  QPopupMenu *edit;
  QPopupMenu *commands;
  QPopupMenu *view;
  int commandsIndicies[8];
  int viewIndicies[3];
  QPopupMenu *help;
  QToolBar *toolbar;
  QToolButton *b1,*b2,*b3,*b4,*b5,*b6,*b7,*b8,*b9;
  QPixmap blankicon;//stepicon, nexticon, continueicon, exiticon;
  MyMultiLineEdit *centralwidget;
  DVisConsole *console;
  DVertSplit *vertsplit;
  DPaintWindow *rtWindow;
  DPaintWindow *ctWindow;
  DScrollWidget* rtGB;
  DScrollWidget* ctGB;
  DDebugController* db;
  char** parsedArray;
  static const int maxParameters = 3;
  LCSubstring* currLCArray;
  int currKN;
  RuleBlockList* _break_rbList;
  RuleBlockList* _curr_rbList;
  int currArraySize;
  int* currBA;
  int* currCA;
  DYesNoDialogue *q1;
  DYesNoDialogue *q2;
  DYesNoDialogue *q3;
  DSymbolVisList *_svl;
  DStackWindow* _stackWindow;
  char* currStack;
  DTransformationList* _transList;

};

#endif
