//////////////////////////////////////////////////////////////////////////////
//
//  NAME: Joon S.  Lee (jl)
//  FILE: PCEqn.H
//  PROJ: COKO
//  DATE: 10/9/96
//
// SYNOPSIS
//  Class definitions for Eqns.
//
//////////////////////////////////////////////////////////////////////////////
 
#ifndef _PCEQN_H_
#define _PCEQN_H_

#include "PCPattern.H"

class CEqn {
protected:
  char * _lhs;
  // stores left hand side of an equation ( variable name )

  CPattern * _rhs;
  // stores right hand side of an equation ( pattern )

  int _evalFlag;
  // indicates whether an equation has been evaluated or not
  // if an equation is not evaluated then _evalFlag is 0.
  // Otherwise, _evalFlag is 1.
  // We need _evalFlag to undo any changes that caused by evaluating equations.
  // We need to know which equation is evaluated and which is not.

  void SetEF(void);       
  // Set _evalFlag ( _evalFlag = 1 )

  void UnsetEF(void);     
  // Unset _evalFlag ( _evalFlag = 0 )

  int key_number;

public:

  CEqn(char*, CPattern*); 
  // Constructor

  ~CEqn(); 
  // Destructor

  CState* Exec(CState *);  
  // What to do when executed

  CState* undoDEP(CState *);  
  // undo any changes on dependency list

  void endOfEval(void) {UnsetEF();}
  // evaluation of equation ended. unset _evalFlag as it was.


  //DJA SUMMER 2001:
  //Added rep_string, representation, number_self, output_self, get_key_number, and ExecDebug methods

  CState* ExecDebug(CState *, DDebugController *);  
  // What to do when executed in debug mode

  char* rep_string ();
  //returns the string representation of only the current 
  //node of the COKO tree
  
  char* representation ();
  //returns string representation of COKO tree

  char* representationWDI(DDebugController *);
  //returns string representation of COKO tree
  //but also update the debugger with the location (row and
  //column) of each breakable statement's substring

  int number_self(int);
  //assigns an integer to the current node for use in the
  //debugger for the breakOptions command

  void output_self(DDebugController *db);
  //output key_number and rep_string for debugger
  //for breakOptions command

  virtual int get_key_number();
  //returns the value stored in the protected variable key_number;
  
};

#endif
