//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Kee-Eung Kim (kek), Joon S. Lee (jl)
//  FILE: PCMatchEachStmt.C
//  PROJ: COKO
//  DATE: 5/5/97
//
// SYNOPSIS
//  Method definitions for CMatchEachStmt class.
//  
//////////////////////////////////////////////////////////////////////////////

#include "PCMatchEachStmt.H"
#include "PCMulMatchStmt.H" // changed on Feb 23

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CMatchEachStmt
// Parameters: (CChildPattern*, CStmt*)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CMatchEachStmt::CMatchEachStmt(CChildPattern *p, CStmt *body)
{
  _first = p;
  _second = body;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CMatchEachStmt
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CMatchEachStmt::~CMatchEachStmt(void)
{
  if(_first) { delete _first; _first = NULL; }
  if(_second) { delete _second; _second = NULL; }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters:	(CState*)
//
// What to do when executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CMatchEachStmt::Exec(CState *s)
{
  if(_debug_)
	cout << "-- MatchEachStmt Begins" << endl;

  // Save the pattern we were using and update it with the new one
  // specified in this match each child statement. "CurrentPattern"
  // will be used to build modified kola query right after returning
  // from rule invokes.
  CPattern *old_pattern = s->CurrentPattern();
  s->SetCurrentPattern(_first);

  s->INew(); // Clear out the top of identifier stack by pushing an empty
             // element.

  // Every iteration step will match identifier with current root's next
  // matching child. 
  for(s = _first->initChildMatch(s), s = _first->nextChildMatch(s);
	  s->ISuccess(); // Loop while nextChildMatch is successful
	  s->IOld(), s->INew(), s = _first->nextChildMatch(s)) 
	{
	  s = _second->Exec(s);
	}

  s->IOld(); // Restore the old identifier environment.
  s->SetCurrentPattern(old_pattern); // Restore the old pattern.
  
  if(_debug_)
	cout << "-- MatchEachStmt Ends" << endl;
  
  return s;
}




