//////////////////////////////////////////////////////////////////
//
//  NAME: Joon S. Lee (jl)
//  
//  FILE: PCRule.H
//  PROJ: COKO
//  DATE: 6/96
// 
//  SYNOPSIS
//  This class is for Rule Declaration/Invoke statements.
//
/////////////////////////////////////////////////////////////////

#ifndef _PCRULE_H_
#define _PCRULE_H_

#include <PCState.H>
#include "KONameUnion.H"
#include "KGlobals.H"
#include "PCPattern.H"
#include "KONonType.H"

class CRule
{
 public:
  CRule (CPattern*, CPattern*);
  //  Constructor for rule representation. 
  //  Takes two arguments that are patterns of
  //  lhs and rhs, respectively. 

  CRule (char*, CPattern*, CPattern*);
  //  Constructor for rule representation. 
  //  Takes three arguments that are name, patterns of
  //  lhs and rhs, respectively. 

  virtual ~CRule();
  // Destructor
 
  virtual CState *Exec(CState*);
  //  Executes the given statement.
  //  Takes the state of the program before the statement
  //  is executed, and then returns the state of the 
  //  program after the statement is executed.

  CState *InvExec(CState*);
  //  Executes the given statement "in reverse".
  //  Takes the state of the program before the statement
  //  is executed, and then returns the state of the 
  //  program after the statement is executed.


  //DJA SUMMER 2001:
  //Added rep_string, representation, get_name, get_rep, InvExecDebug, and ExecDebug methods
  
  char* rep_string ();
  //returns the string representation of only the current 
  //node of the COKO tree
  
  char* representation ();
  //returns string representation of rule

  CState *ExecDebug(CState *, DDebugController *, CStmt*);
  //what to do when rule is executed in debug mode

  CState *InvExecDebug(CState *, DDebugController *, CStmt*);
  //what to do when inverse rule is executed in debug mode

  char* get_name();
  //returns a copy of what is stored in the _name instance variable
  
  char* get_rep();
  //returns a copy of the non-name representation of the rule

 protected:

  char* _name;
  CPattern* _lhs;
  CPattern* _rhs;
};

#endif
