//////////////////////////////////////////////////////////////////////////////
//
// NAME: Joon S. Lee (jl)
//
// FILE: PCStmt.C
// PROJ: COKO
// DATE: 5/5/97
//
// SYNOPSIS
//  Class definition for CStmt class.
//
//////////////////////////////////////////////////////////////////////////////

#include "PCStmt.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: updateDEP
// Parameters:	(char*, CState*, CIEnvStackDataType*, int)
//
// updates variable dependancy list
//         								    
//////////////////////////////////////////////////////////////////////////////

int _debug_=0;
int _cldbg_=0;
int _clcokodb_=0;
int _vcokodb_=0;

CState* CStmt::updateDEP(char* key, CState* s,CIEnvStackDataType* startingPt, int startFlag)
{

  CIEnvStackDataType* startingPt_next;
  
  if (startFlag == 1)
	startingPt = s->IEnv()->getData();
   
  if (startingPt)
	startingPt_next = startingPt->_next;
  else
	startingPt_next = NULL;


  CPattern* tmp_pattern;

  if (!s->IEnv()->top())
	{
	  cout << "Error @ CStmt : updateDEP" << endl;
	  cout << "Stack is empty error" << endl;
	  exit(1);
	}
  else
	{
	  
	  if (_debug_)
		cout << "updating begin with initial variable " << key << endl;
	  
	  CIEnvNodeType * tmpNode = s->IEnv()->findNodeFromHere(key, startingPt);
	  
	  if (tmpNode)
		{
		  tmpNode->initIteratorDEP();
		  char* tmpchar = tmpNode->getValueDEP();
		  
		  if (_debug_)
			cout << key << " has " << tmpchar << endl;
		  
		  if (tmpchar) 
			{
			  tmp_pattern = s->GetPatternFromHere(tmpchar, startingPt_next);
			  if (_debug_)
				cout << tmpchar << " has pattern " << tmp_pattern->getNT()->representation(); 
			  if (tmp_pattern)
				{
				  CIdent * matchNode = s->IFindFromHere(tmpchar, startingPt_next);
				  
				  s = tmp_pattern->buildFromHere(s, startingPt);
				  
				  s->IModifyFromHere(tmpchar, 
									 s->Store(),
									 matchNode->Which(),
									 startingPt_next);
				}
			  else 
				{
				  cout << "Error @ CStmt : updateDEP" << endl;
				  cout << "Pattern is empty error" << endl;
				  exit(1);
				}
			  
			  s = updateDEP(tmpchar, s, startingPt_next, 0);
			}
  		  while (tmpchar)
  			{
  			  tmpchar = tmpNode->getValueDEP();
  			  if (tmpchar)
  				{
 				  tmp_pattern = s->GetPatternFromHere(tmpchar, startingPt_next);
  				  if (tmp_pattern)
  					{
  					  CIdent * matchNode = s->IFindFromHere(tmpchar, startingPt_next);
  					  s = tmp_pattern->buildFromHere(s, startingPt);
  					  s->IModifyFromHere(tmpchar, 
 										 s->Store(),
 										 matchNode->Which(),
 										 startingPt_next);
  					}
 				  s = updateDEP(tmpchar,s, startingPt_next, 0);
  				}
		  	}
		}
	}
  return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: number_self
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Uses start parameter to assign key_number data_member and 
// returns current key-number
//         								    
//////////////////////////////////////////////////////////////////////////////

int CStmt::number_self(int start) {
  key_number = start;
  return (start + 1);
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: output_self
// Parameters:	(DDebugController *)
//
// ADDED DJA SUMMER 2001
// 
// Default output for statements to output representation of itself for the
// debugger list command (key_number and rep_string). This assumes that the 
// statement contains no sub-statements and is a breakable statement.
// Otherwise, this method must be overridden.
//         								    
//////////////////////////////////////////////////////////////////////////////

void CStmt::output_self(DDebugController *db)
{
  char* rep = rep_string();
  db->format_output(key_number, rep);
  delete [] rep;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: get_key_numbner
// Parameters:	()
//
// ADDED DJA SUMMER 2001
// Returns value stored in the key_number data member.
//         								    
//////////////////////////////////////////////////////////////////////////////

int CStmt::get_key_number()
{
  return key_number;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: get_indent_string
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Returns a string containing an indent number of the ' ' character.
// Used by representation methods for indentation.
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CStmt::get_indent_string(int indent)
{
  char* indent_string = new char[indent + 1];
  for (int i = 0; i < indent; i++)
    indent_string[i] = ' ';
  indent_string[indent] = '\0';
  return indent_string;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// 
// returns the string representation of the COKO subtree
// beginning at this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CStmt::representation(int indent)
{
  char* temp = new char[strlen("ERROR: REPRESENTATION METHOD NOT PROGRAMMED FOR THIS STATEMENT") + 1];
  sprintf (temp, "ERROR: REPRESENTATION METHOD NOT PROGRAMMED FOR THIS STATEMENT");
  return temp;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representationWDI
// Parameters:	(int, DDebugController*)
//
// ADDED DJA SUMMER 2001
// 
// returns the string representation of the COKO subtree
// beginning at this object but includes debugging info about string
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CStmt::representationWDI(int indent, DDebugController* db)
{
  char* temp = new char[strlen("ERROR: REPRESENTATIONWDI METHOD NOT PROGRAMMED FOR THIS STATEMENT") + 1];
  sprintf (temp, "ERROR: REPRESENTATIONWDI METHOD NOT PROGRAMMED FOR THIS STATEMENT");
  return temp;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_string
// Parameters:	()
//
// ADDED DJA SUMMER 2001
// 
// returns the string representation of only the current 
// node of the COKO tree
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CStmt::rep_string()
{
  char* temp = new char[strlen("ERROR: REP_STRING METHOD NOT PROGRAMMED FOR THIS STATEMENT") + 1];
  sprintf (temp, "ERROR: REP_STRING METHOD NOT PROGRAMMED FOR THIS STATEMENT");
  return temp;
}
