//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon S. Lee (jl)
//  FILE: PCTopDownStmt.C
//  PROJ: COKO
//  DATE: 6/15/97
//
// SYNOPSIS
//  Method definitions for CTopDownStmt class.
//  
//////////////////////////////////////////////////////////////////////////////

#include "PCTopDownStmt.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CTopDownStmt
// Parameters: (CStmt *)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CTopDownStmt::CTopDownStmt(CStmt * stmt)
{
  _stmt = stmt;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CTopDownStmt
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CTopDownStmt::~CTopDownStmt(void)
{
  if(_stmt) { delete _stmt; _stmt = NULL; }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters:	(CState*)
//
// What to do when executed.
// Execute _stmt on Every node of the tree (current tree.)
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CTopDownStmt::Exec(CState *s)
{
  NonType * root = NULL;
  NonType * curr_child = NULL;
  int curr_which = 1;
  int success = 0;

  s = _stmt->Exec(s);

  success = s->ISuccess() || success;

  root = s->Store()->Copy_kTree();
  root->initCurrChild();
  curr_child = root->nextChild();
  while (curr_child)
	{
	  curr_child = curr_child->Copy_kTree();
	  s->SetStore(curr_child);
	  Exec(s); // recursion

	  success = s->ISuccess() || success;

	  NonType* result_child = s->Store()->Copy_kTree();

	  root->replaceChild(result_child, curr_which);
	  curr_child = root->nextChild();
	  curr_which++;
	}
  s->SetStore(root); // Restore old root

  // set success value.
  s->SetSuccess(success);

  return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ExecDebug
// Parameters:	(CState*, DDebugController*)
//
// What to do when executed.
// Execute _stmt on Every node of the tree (current tree.)
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CTopDownStmt::ExecDebug(CState *s, DDebugController *db)
{
  if (db->really_execute()) {
    NonType * root = NULL;
    NonType * curr_child = NULL;
    int curr_which = 1;
    int success = 0;
    
    s = _stmt->ExecDebug(s, db);
    
    success = s->ISuccess() || success;
    
    root = s->Store()->Copy_kTree();
    root->initCurrChild();
    curr_child = root->nextChild();
    while (curr_child)
      {
	db->histPush(root->Copy_kTree(), 1, curr_which, NULL, NULL, NULL);
	curr_child = curr_child->Copy_kTree();
	s->SetStore(curr_child);
	ExecDebug(s, db); // recursion
	
	success = s->ISuccess() || success;
	
	NonType* result_child = s->Store()->Copy_kTree();
	
	root->replaceChild(result_child, curr_which);
	curr_child = root->nextChild();
	curr_which++;
	db->histPop();
      }
    s->SetStore(root); // Restore old root
    
    // set success value.
    s->SetSuccess(success);
  }
  return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_string
// Parameters:	(void)
//
// Returns the string representation of only the current 
// node of the COKO tree
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CTopDownStmt::rep_string ()
{
  char *res = new char [strlen ("TD") + 1];
  sprintf (res, "TD");
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Returns string representation of COKO tree starting from this statement
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CTopDownStmt::representation (int indent)
{
  char *stmt;
  if (_stmt -> is_leaf())
    stmt = _stmt -> representation (indent);
  else {
    char *stmt_orig = _stmt -> representation (indent);
    stmt = new char[strlen(stmt_orig) + strlen("{}") + 1];
    sprintf (stmt, "{%s}", stmt_orig);
    delete[] stmt_orig;
  }
  char *rep = rep_string();
  char *res = new char [strlen (stmt) + strlen (rep) + strlen (" ") + 1];
  sprintf (res, "%s %s", rep, stmt);
  delete [] stmt;
  delete [] rep;
  return res; 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representationWDI
// Parameters:	(int, DDebugController *)
//
// ADDED DJA SUMMER 2001
// returns the string representation of this COKO RuleBlock, but
// at the same time updates an array in db that locates sub-strings for
// each key number
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CTopDownStmt::representationWDI (int indent, DDebugController *db)
{
  char *rep = rep_string();
  int currLine = db->getLine();
  int currCol = db->getCol();
  //db->updateArray(key_number-1, currLine, currCol, currLine, currCol + strlen(temp)+1);
  db->setCol(currCol + strlen(rep) + 1);
  char *stmt;
  if (_stmt -> is_leaf())
    stmt = _stmt -> representationWDI (indent,db);
  else {
    db->setCol(currCol+strlen(rep)+2);
    char *stmt_orig = _stmt -> representationWDI (indent, db);
    stmt = new char[strlen(stmt_orig) + strlen("{}") + 1];
    sprintf (stmt, "{%s}", stmt_orig);
    delete[] stmt_orig;
    currCol = db->getCol();
    db->setCol(currCol+1);
  }
  char *res = new char [strlen (stmt) + strlen (rep) + strlen (" ") + 1];
  sprintf (res, "%s %s", rep, stmt);
  delete [] stmt;
  delete [] rep;
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: number_self
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Uses start parameter to assign key_number data_member, calls number_self 
// on any sub_statements and returns current key-number
//         								    
//////////////////////////////////////////////////////////////////////////////

int CTopDownStmt::number_self (int start)
{
  key_number=0; //TD not a breakable statement
  return (_stmt -> number_self(start));
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: output_self
// Parameters:	(DDebugController *)
//
// ADDED DJA SUMMER 2001
// Outputs current statement in list format (for the debugger) if it is a 
// breakable statement; otherwise just calls output_self on any sub-statements
//         								    
//////////////////////////////////////////////////////////////////////////////

void CTopDownStmt::output_self (DDebugController *db)
{
  //TD is not a breakable statement
  _stmt->output_self(db);
}

int CTopDownStmt::is_leaf()
  //returns whether current statement is a leaf in the COKO tree
{
  //I know that TD isn't really a leaf, but for the purposes of the
  //representation method which uses is_leaf, TD is just one statement
  //like a leaf
  return 1;
}
