/*
 * Copyright 1995, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *	Name:
 *	Organization:
 *	Address (postal and/or electronic):
 *
 * To:
 *	Software Librarian
 *	Computer Science Department, Box 1910
 *	Brown University
 *	Providence, RI 02912
 *
 *		or
 *
 *	brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/*
//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon S. Lee (jl@cs.brown.edu)
//  FILE: PCoko.y
//
//  PROJ: COKO Compiler
//  DATE: 8/97                                                          
//
//////////////////////////////////////////////////////////////////////////////
*/

%token <sval> PIDENT FIDENT OIDENT BIDENT TIDENT IDENT STRINGCON INTCON FLOATCON
%token PDC FDC ODC BDC IDC TDC BEG END ERROR
%token EQUAL NOTEQUAL LT GT LE GE NE
%token OPLUS CONST CURRY INV XTIMES TIMES EXISTS FORALL EX FA 
%token INS SORT LSJOIN RSJOIN PEQUAL AND OR
%token ID SET ONE TWO SHIFTL SHIFTR CHOOSE FLATTEN 
%token CIRC PLUS MINUS CON UNI INT DIF PFOLD FOLD ITERATE ITER PCON JOIN NJOIN UNNEST  
%token EMPTY NIL FINVOKE PINVOKE TRUEv FALSEv ADD SUB MUL DIV SINGLETON TID
%token RANGLE LANGLE AVG SUM COUNT MIN MAX LEX
%token PROPERTY IMPL BAND BOR INFERS NOT PERIOD OID INTER ODIF FINV ABS MOD MULT DIV IF FI ELSE FTHEN
%token DPO DPC KSTRING KFLOAT KCHAR INVERSE

%right FINVOKE PINVOKE
%nonassoc TRUEv FALSEv
%nonassoc LOW
%nonassoc HIGH
%left '-' '+' ODIF
%left '*' '/'
%left UNION INTER XTIMES DIV MOD MULT TIMES PLUS MINUS CIRC OPLUS '&' '|' 
%left BOR BAND
%left FINV INV
%left PEQUAL LESS GREATER LE GE NE

%type <PRB> Program
%type <PS> Exp SExp PBody PStmt ArgList Pred PrimP FormP Func PrimF FormF Obj PrimO FormO Bool PrimB FormB Type

%start Program

%{
#include <iostream.h>
#include <string.h>
#include "PParser.h"
#include <algo.h>
#include <fstream.h>
#include <stdlib.h>

extern int lineno;
extern char CKtext[];

extern int CKlex ();
int CKerror(char *);

static char *buffer;
static int strlength;
static char *PBname;
PPropertyBlock *theBlock;
static PAIdentSet pr_includes;
static PAIdentSet ko_includes;
static PAIdentSet ty_includes;
 static char* myName;
static int dontCareCount;

%}

%union {
  PPropertyBlock *PRB;
  PCode *PS;
  char *sval;
}


%%


/********************************************************************************
 ********************************************************************************

   PropertyBlocks

   What Constitutes a COKO Property Block?

 *******************************************************************************
********************************************************************************/

/*********************************************************************************
   PropertyBlock

**********************************************************************************/

Program         :  PROPERTY IDENT 
                
                { 
		  myName = new char[ strlen($2) + 1 ];
		  sprintf(myName, "%s", $2)
		  //PBname = new char[ strlen( string($2) ) ];
		  //sprintf(PBname, "%s", string($2));
		    		} 

                    OptInfer BEG PBody END

                    {
		      PAIdentSet includes;
		      for(PAIdentSet::iterator i = pr_includes.begin(); i != pr_includes.end(); i++) {

			buffer = new char[ strlen( "#include \"") + strlen(i->chars()) + strlen( "Property.h\"" ) + 1 ];
			sprintf(buffer, "#include \"%sProperty.h\"", i->chars());
			includes.insert(string(buffer));
			
		      }
		      
		      for(PAIdentSet::iterator i = ko_includes.begin(); i != ko_includes.end(); i++) {
			buffer = new char[ strlen( "#include \"KO") + strlen(i->chars()) + strlen( ".H\"" ) + 1 ];
			sprintf(buffer, "#include \"KO%s.H\"", i->chars());
			includes.insert(string(buffer));
			
		      }
		     
		      for(PAIdentSet::iterator i = ty_includes.begin(); i != ty_includes.end(); i++) {

			buffer = new char[ strlen( "#include \"TYT") + strlen(i->chars()) + strlen( ".H\"" ) + 1 ];
			sprintf(buffer, "#include \"TYT%s.H\"", i->chars());
			includes.insert(string(buffer));
			
		      }
					  $$ = theBlock = new PPropertyBlock(string(myName), includes, $6);
					}

                ;

/********************************************************************************
																				 
OptInfer
																				 
********************************************************************************/

OptInfer        : INFERS 

                   {
		   }

                   InfersClause

                   {
                   }

| //IDENT

                   {	
		   }

                ;

/********************************************************************************
																				 
InfersClause
																				 
********************************************************************************/

InfersClause    : InfersClause ',' IDENT

                  {
		    pr_includes.insert(string($3));
                  }

                | IDENT

                   {
		     pr_includes.insert(string($1));	
		   }

                ;

/********************************************************************************
																				 
PBody
																				 
********************************************************************************/

PBody           : PStmt PERIOD PBody

                   {
		     $$ = new PCode;
		     strlength = strlen("new OrProperty( ") + strlen($1->code.chars()) + strlen(", ") + strlen($3->code.chars()) + strlen( " )" ) + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new OrProperty( %s, %s )", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);
					
				   }

                | PStmt PERIOD

                   {
		     $$ = $1	;
		   }              

                ;

/********************************************************************************
																				 
PStmt
																				 
********************************************************************************/

// Marina Summer 2001 changed grammar from: Exp IMPL SExp to Exp IMPL IDENT '(' ArgList ')'
// and from: SExp to IDENT '(' ArgList ')'.
// This was necessary in order to differntiate an expression on the RHS from one on the LHS.
// An expression on the RHS is just what is inferred if the expression on the LHS is true -
// this change was necessary so that MatchProperties and SchemaProperties, etc. are not 
// generated for expressions on the RHS.

PStmt           : Exp IMPL IDENT '(' ArgList ')'

                   {
			$$ = new PCode;
			strlength = strlen("new AndProperty( new MatchProperty( new NonTypeArgs(1, ") + strlen($5->code.chars()) + 
			  strlen(", NULL) ), ") + strlen($1->code.chars()) + strlen( " )") + 1;
			buffer = new char[strlength];
			sprintf(buffer, "new AndProperty( new MatchProperty( new NonTypeArgs(1, %s, NULL) ), %s )", $5->code.chars(), 
				$1->code.chars());	
			$$->code = string(buffer);	
		   }

                | IDENT '(' ArgList ')'

                   {
		     $$ = new PCode;
		     strlength = strlen("new MatchProperty( new NonTypeArgs( 1, ") + strlen($3->code.chars()) + strlen( ", NULL ) )") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new MatchProperty( new NonTypeArgs( 1, %s, NULL ) )", $3->code.chars());	
		     $$->code = string(buffer);
					
		   }

                ;

/********************************************************************************
																				 
SExp
																				 
********************************************************************************/

SExp            : IDENT '(' ArgList ')'

                   {
		     $$ = new PCode;
		     if ( pr_includes.find($1) != pr_includes.end() || string(myName) == string($1) ){
		       strlength = strlen("new BoundProperty<Property>(new NonTypeArgs( 1, , NULL ))") + strlen($1) + 
			 strlen( $3->code.chars()) + 1;
		       buffer = new char[strlength];
		       sprintf(buffer, "new BoundProperty<%sProperty>(new NonTypeArgs( 1, %s, NULL ))", $1, $3->code.chars());
		       $$->code = string(buffer);
		     }
		     else{
		       strlength = strlen("new SchemaProperty(\"\", new NonTypeArgs( 1, , NULL ) )") + strlen($1) + strlen($3->code.chars()) + 1;
		       buffer = new char[strlength];
		       sprintf(buffer, "new SchemaProperty(\"%s\", new NonTypeArgs( 1, %s, NULL ) )", $1, $3->code.chars());
		       $$->code = string(buffer);
		     } 	
		   }

                ;

/********************************************************************************
																				 
ArgList
																				 
********************************************************************************/

ArgList         : ArgList ',' Func

                  {
				$$ = new PCode;
				strlength = strlen($1->code.chars()) + strlen(", ") + strlen($3->code.chars()) + 1;
				buffer = new char[strlength];
				sprintf(buffer, "%s, %s", $1->code.chars(), $3->code.chars());
				$$->code = string(buffer);	
				  }

                | ArgList ',' Pred 

                   {
		     $$ = new PCode;
		     strlength = strlen($1->code.chars()) + strlen(", ") + strlen($3->code.chars()) + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "%s, %s", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);
				   }
                

                | ArgList ',' Obj 

                   {
		     $$ = new PCode;
		     strlength = strlen($1->code.chars()) + strlen(", ") + strlen($3->code.chars()) + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "%s, %s", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);	
				   }
                

                | ArgList ',' Bool 

                   {
		     $$ = new PCode;
		     strlength = strlen($1->code.chars()) + strlen(", ") + strlen($3->code.chars()) + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "%s, %s", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);	
				   }

                | ArgList ',' Type 

                   {
		     $$ = new PCode;
		     strlength = strlen($1->code.chars()) + strlen(", ") + strlen($3->code.chars()) + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "%s, %s", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);		
				   }

                | Func 

                   {
		     $$ = $1;		 
				   }

                | Pred 

                   {
		     $$ = $1;
				   }

                | Obj 

                   {
		     $$ = $1;
				   }

                | Bool

                   {
		     $$ = $1;
				   }

                | Type

                   {
		     $$ = $1;
				   }

                ;

/********************************************************************************
																				 
Exp
																				 
********************************************************************************/
// MARINA  changed grammar from :  Exp AND SExp to Exp AND Exp, and from Exp OR SExp to Exp OR Exp

Exp             : Exp AND Exp    

                   {
		     $$ = new PCode;
		     strlength = strlen("new AndProperty( ") + strlen($1->code.chars()) + strlen(", ") + strlen($3->code.chars()) + 
		       strlen(" )") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new AndProperty( %s, %s )", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);
				   }

                | Exp OR Exp

                   {
		     $$ = new PCode;
		     strlength = strlen("new OrProperty( ") + strlen($1->code.chars()) + strlen(", ") + strlen($3->code.chars()) + 
		       strlen(" )") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new OrProperty( %s, %s )", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);		
				   }

                | NOT '(' Exp ')'

                   {
		     $$ = new PCode;
		     strlength = strlen("new NotProperty( ") + strlen($3->code.chars()) + strlen( " )") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new NotProperty( %s )", $3->code.chars());		 
		     $$->code = string(buffer);
				   }

                | SExp

                   {
		     $$ = $1;
		   }

                |  Func '=' Func

                   {
				
				   }

                |  Pred '=' Pred

                   {
					
				   }

                |  Obj '=' Obj

                   {
				   }

                |  Bool '=' Bool

                   {
				
				   }

                |  Type '=' Type

                   {
					
				   }

                ;


/********************************************************************************
Pred
																				 
 Predicate Rule Expressions.
 Note that predicate variables (PIDENT) are assumed to be of the form
																				 
	p, p', p'', ...
	p1, p2, p3, ...
	q, q', q'', ...
	q1, q2, q3, ...
	r, r', r'', ...
	r1, r2, r3, ...
																				 
********************************************************************************/

Pred            :  PrimP { $$ = $1; }

                |  FormP 

                   { $$ = $1; }

                |  PIDENT 

                   {
		     $$ = new PCode;
		     ko_includes.insert(string("PVar"));
		     strlength = strlen("new PVar(\"") + strlen($1) + strlen("\")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PVar(\"%s\")", $1);
		     $$->code = string(buffer);
				   }  

                |  PDC    

                   {
			ko_includes.insert( string("PVar"));
					  
			$$ = new PCode;
			char count[10];
			sprintf(count, "%d", dontCareCount++);
			strlength = strlen("new PVar(\"_P") + strlen(count) + strlen("\")") + 1;
			buffer = new char[strlength];
			sprintf(buffer, "new PVar(\"_P%s\")", count);
			$$->code = string(buffer);	
		   }

                ;

PrimP           :  EQUAL      

                   {
		     ko_includes.insert( string("PEqual"));
		     $$ = new PCode;
		     strlength = strlen("new PEqual") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new PEqual");
		     $$->code = string(buffer);
				   }

                |  NOTEQUAL

                   {
		     ko_includes.insert(string("PNotEqual"));
		     $$ = new PCode;
		     strlength = strlen("new PNotEqual") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new PNotEqual");
		     $$->code = string(buffer);		
				   }

                |  LESS       

                   { 
		     ko_includes.insert(string("PLess"));
		     $$ = new PCode;
		     strlength = strlen("new PLess") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new PLess");
		     $$->code = string(buffer);		
				   }

                |  GREATER

                   {
		     ko_includes.insert(string("PGreater"));
		     $$ = new PCode;
		     strlength = strlen("new PGreater") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new PGreater");
		     $$->code = string(buffer);		
				   }                    

                |  LE     

                   { 
		  ko_includes.insert(string("PLessEqual"));
		  $$ = new PCode;
		  strlength = strlen("new PLessEqual") + 1;
		  buffer = new char[strlength];
		  strcpy(buffer, "new PLessEqual");
		  $$->code = string(buffer);			
				   }

                |  GE     

                   { 
		
		     ko_includes.insert(string("PGreaterEqual"));
		     $$ = new PCode;
		     strlength = strlen("new PGreaterEqual") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new PGreaterEqual");
		     $$->code = string(buffer);		
				   }

                ;

FormP           :  Pred OPLUS Func

                   { 
		     ko_includes.insert( string("PCombine"));
		     $$ = new PCode;
		     strlength = strlen("new PCombine(") + strlen($1->code.chars())
		       + strlen(", ") + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PCombine(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);		
				   }

                |  LEX '(' Pred ',' Pred ')'

                   { 
		     ko_includes.insert( string("PLex"));
		     $$ = new PCode;
		     strlength = strlen("new PLex(") + strlen($3->code.chars())
		       + strlen(", ") + strlen($5->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PLex(%s, %s)", $3->code.chars(), $5->code.chars());
		     $$->code = string(buffer);		
				   }

                |  Pred '&' Pred

                   { 
		     ko_includes.insert(string("PConjunct"));
		     $$ = new PCode;
		     strlength = strlen("new PConjunct(") + strlen($1->code.chars())
		       + strlen(", ") + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PConjunct(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);		
				   }

                |  Pred '|' Pred

                   { 
		     ko_includes.insert( string("PDisjunct"));
		     $$ = new PCode;
		     strlength = strlen("new PDisjunct(") + strlen($1->code.chars())
		       + strlen(", ") + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PDisjunct(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);		
				   }

                |  NOT '(' Pred ')'   

                   { 
		     ko_includes.insert( string("PNegate"));
		     $$ = new PCode;
		     strlength = strlen("new PNegate(") + strlen($3->code.chars())
		       + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PNegate(%s)", $3->code.chars());
		     $$->code = string(buffer);		
				   }

                |  CONST '(' Bool ')'     

                   { 
		     ko_includes.insert(string("PConst"));
		     $$ = new PCode;
		     strlength = strlen("new PConst(") + strlen($3->code.chars())
		       + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PConst(%s)", $3->code.chars());
		     $$->code = string(buffer);			
				   }

                |  CURRY '(' Pred ',' Obj ')' 

                   { 
		     ko_includes.insert(string("PCElt"));
		     $$ = new PCode;
		     strlength = strlen("new PCElt(") + strlen($3->code.chars())
		       + strlen(", ") + strlen($5->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PCElt(%s, %s)", $3->code.chars(), $5->code.chars());
		     $$->code = string(buffer);	
				   }

                |  Pred INV   

                   { 
		     ko_includes.insert(string("PInv"));
		     $$ = new PCode;
		     strlength = strlen("new PInv(") + strlen($1->code.chars())
		       + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PInv(%s)", $1->code.chars());
		     $$->code = string(buffer);		
				   }

                |  Pred TIMES Pred    

                   { 
		     ko_includes.insert( string("PTimes"));
		     $$ = new PCode;
		     strlength = strlen("new PTimes(") + strlen($1->code.chars())
		       + strlen(", ") + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PTimes(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);		
				   }

                |  Pred XTIMES Pred    

                   { 
		     ko_includes.insert( string("PTimes"));
		     $$ = new PCode;
		     strlength = strlen("new PTimes(") + strlen($1->code.chars())
		       + strlen(", ") + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PTimes(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);		
				   }

                |  EXISTS '(' Pred ')'    

                   { 
		     ko_includes.insert(string("PExists"));
		     $$ = new PCode;
		     strlength = strlen("new PExists(") + strlen($3->code.chars())
		       + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PExists(%s)", $3->code.chars());
		     $$->code = string(buffer);		
				   }

                |  FORALL '(' Pred ')'    

                   { 
		     ko_includes.insert(string("PForall"));
		     $$ = new PCode;
		     strlength = strlen("new PForall(") + strlen($3->code.chars())
		       + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PForall(%s)", $3->code.chars());
		     $$->code = string(buffer);			
				   }

                |  EX '(' Pred ')'    

                   { 
		     ko_includes.insert( string("PCEX"));
		     $$ = new PCode;
		     strlength = strlen("new PCEX(") + strlen($3->code.chars())
		       + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PCEX(%s)", $3->code.chars());
		     $$->code = string(buffer);	
				   }

                |  FA '(' Pred ')'        

                   { 
		     ko_includes.insert( string("PCFA"));
		     $$ = new PCode;
		     strlength = strlen("new PCFA(") + strlen($3->code.chars())
		       + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new PCFA(%s)", $3->code.chars());
		     $$->code = string(buffer);		
				   }

                |  '(' Pred ')'   { $$ = $2; } 

                ;

/********************************************************************************
 Func
																			 
	Function Rule Expressions.
																				 
	Note that function variables (FIDENT) are assumed to be of the form
																				 
	 f, f', f'', ...
	 f1, f2, f3, ...
	 g, g', g'', ...
	 g1, g2, g3, ...
	 h, h', h'', ...
	 h1, h2, h3, ...
																				 
********************************************************************************/

Func            :  PrimF  { $$ = $1; }

                |  FormF  

                   { $$ = $1; }

                |  FIDENT 

                   {
		     ko_includes.insert( string("FVar") );
		     $$ = new PCode;
		     strlength = strlen("new FVar(\"") + strlen($1) + strlen("\")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FVar(\"%s\")", $1);
		     $$->code = string(buffer);		 
		   }

                |  FDC    

                   { 
			 ko_includes.insert(string("FVar"));
			 $$ = new PCode;
			 char count[10];
			 sprintf(count, "%d", dontCareCount++);
					  
			 strlength = strlen("new FVar(\"_F") + strlen(count) + strlen("\")") + 1;
			 buffer = new char[strlength];
			 sprintf(buffer, "new FVar(\"_F%s\")", count);
			 $$->code = string(buffer);	
		   } 

                ;

PrimF           :  ID 

                   { 
		     ko_includes.insert(string("FID"));
		     $$ = new PCode;
		     strlength = strlen("new FID") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FID");
		     $$->code = string(buffer);
				   } 

                |  TID

                    { 
		      ko_includes.insert( string("FTID"));
                      $$ = new PCode;
                      strlength = strlen("new FTID") + 1;
                      buffer = new char[strlength];
                      strcpy(buffer, "new FTID");
                      $$->code = string(buffer);		  
                    }

                |  ABS

                    { 
		      ko_includes.insert(string("FAbs"));
                      $$ = new PCode;
                      strlength = strlen("new FAbs") + 1;
                      buffer = new char[strlength];
                      strcpy(buffer, "new FAbs");
                      $$->code = string(buffer);							  
                    }

                |  AVG

                   { 
		     ko_includes.insert( string("FAvg"));
		     $$ = new PCode;
		     strlength = strlen("new FAvg") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FAvg");
		     $$->code = string(buffer);
				   }

                |  MOD

                   { 
		     ko_includes.insert( string("FMod"));
		     $$ = new PCode;
		     strlength = strlen("new FMod") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FMod");
		     $$->code = string(buffer);		
				   }

                |  INVERSE

                   { 
		     ko_includes.insert(string("FInv"));
		     $$ = new PCode;
		     strlength = strlen("new FInv") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FInv");
		     $$->code = string(buffer);		
				   }

                |  SUM

                   { 
		     ko_includes.insert( string("FSum"));
		     $$ = new PCode;
		     strlength = strlen("new FSum") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FSum");
		     $$->code = string(buffer);		
				   }

                |  COUNT

                   { 
		     ko_includes.insert(string("FCount"));
		     $$ = new PCode;
		     strlength = strlen("new FCount") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FCount");
		     $$->code = string(buffer);	
				   }

                |  MAX

                   { 
		     ko_includes.insert( string("FMax"));
		     $$ = new PCode;
		     strlength = strlen("new FMax") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FMax");
		     $$->code = string(buffer);		
				   }

                |  MIN

                   { 
		     ko_includes.insert( string("FMin"));
		     $$ = new PCode;
		     strlength = strlen("new FMin") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FMin");
		     $$->code = string(buffer);	
				   }

                |  ADD

                   { 
		     ko_includes.insert( string("FAdd"));
		     $$ = new PCode;
		     strlength = strlen("new FAdd") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FAdd");
		     $$->code = string(buffer);	
				   }

                |  SUB

                   { 
		     ko_includes.insert(string("FMinus"));
		     $$ = new PCode;
		     strlength = strlen("new FMinus") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FMinus");
		     $$->code = string(buffer);
				   }

                |  MUL

                   { 
		     ko_includes.insert( string("FMult"));
		     $$ = new PCode;
		     strlength = strlen("new FMult") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FMult");
		     $$->code = string(buffer);
				   }

                |  DIV

                   { 
		     ko_includes.insert( string("FDiv"));
		     $$ = new PCode;
		     strlength = strlen("new FDiv") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FDiv");
		     $$->code = string(buffer);
				   }

                |  SINGLETON

                   {
		     ko_includes.insert(string("FSingleton"));
		     $$ = new PCode;
		     strlength = strlen("new FSingleton") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FSingleton");
		     $$->code = string(buffer);	
				   }

                |  SET    

                   { 
		     ko_includes.insert(string("FSet"));
		     $$ = new PCode;
		     strlength = strlen("new FSet") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FSet");
		     $$->code = string(buffer);	
				   }

                |  ONE

                   {
		     ko_includes.insert(string("F1"));
		     $$ = new PCode;
		     strlength = strlen("new F1") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new F1");
		     $$->code = string(buffer);	
				   }

                |  TWO    

                   { 
		     ko_includes.insert( string("F2"));
		     $$ = new PCode;
		     strlength = strlen("new F2") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new F2");
		     $$->code = string(buffer);	
				   }

                |  SHIFTL 

                   { 
		     ko_includes.insert(string("FShiftL"));
		     $$ = new PCode;
		     strlength = strlen("new FShiftL") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FShiftL");
		     $$->code = string(buffer);
				   }

                |  SHIFTR 

                   { 
		     ko_includes.insert( string("FShiftR"));
		     $$ = new PCode;
		     strlength = strlen("new FShiftR") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FShiftR");
		     $$->code = string(buffer);			
	   }

                |  CHOOSE 

                   { 
		     ko_includes.insert( string("FChoose"));
		     $$ = new PCode;
		     strlength = strlen("new FChoose") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FChoose");
		     $$->code = string(buffer);			
				   }

                |  FLATTEN 

                   { 
		     ko_includes.insert( string("FFlatten"));
		     $$ = new PCode;
		     strlength = strlen("new FFlatten") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FFlatten");
		     $$->code = string(buffer);
		
				   }

                |  INS   

                   {
		     ko_includes.insert(string("FIns"));
		     $$ = new PCode;
		     strlength = strlen("new FIns") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new FIns");
		     $$->code = string(buffer);		
				   }
                ;

FormF           :  CONST '(' Obj ')' 

                   { 
		     ko_includes.insert( string("FConst"));
		     $$ = new PCode;
		     strlength = strlen("new FConst(") + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FConst(%s)", $3->code.chars());
		     $$->code = string(buffer);
				   }

                |  CURRY '(' Func ',' Obj ')' 

                   { 
		     ko_includes.insert(string("FCElt"));
		     $$ = new PCode;
		     strlength = strlen("new FCElt(") + strlen($3->code.chars()) 
		       + strlen(", ") + strlen($5->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FCElt(%s, %s)", $3->code.chars(), $5->code.chars());
		     $$->code = string(buffer);		
				   }

                |  Func TIMES Func 

                   { 
		     ko_includes.insert( string("FTimes"));
		     $$ = new PCode;
		     strlength = strlen("new FTimes(") + strlen($1->code.chars()) 
		       + strlen(", ") + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FTimes(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);
				   }

                |  Func XTIMES Func 

                   { 
		     ko_includes.insert( string("FTimes"));
		     $$ = new PCode;
		     strlength = strlen("new FTimes(") + strlen($1->code.chars()) 
		       + strlen(", ") + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FTimes(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);	
				   }

                |  Func CIRC Func  

                   { 
		     ko_includes.insert( string("FCompose"));
		     $$ = new PCode;
		     strlength = strlen("new FCompose(") + strlen($1->code.chars()) 
		       + strlen(", ") + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FCompose(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);
				   }

                |  Func PLUS Func  

                   { 
		     ko_includes.insert( string("FPlus"));
		     $$ = new PCode;
		     strlength = strlen("new FPlus(") + strlen($1->code.chars()) 
		       + strlen(", ") + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FPlus(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);	
				   }

                |  LANGLE Func ',' Func RANGLE

                   { 
		     ko_includes.insert(string("FPlus"));
		     $$ = new PCode;
		     strlength = strlen("new FPlus(") + strlen($2->code.chars()) 
		       + strlen(", ") + strlen($4->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FPlus(%s, %s)", $2->code.chars(), $4->code.chars());
		     $$->code = string(buffer);	
				   }

                |  CON '(' Pred ',' Func ',' Func ')' 

                   { 
		     ko_includes.insert(string("FCon"));
		     $$ = new PCode;
		     strlength = strlen("new FCon(") + strlen($3->code.chars()) 
		       + strlen(", ") + strlen($5->code.chars()) + strlen(", ") + strlen($7->code.chars())
		       + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FCon(%s, %s, %s)", $3->code.chars(), 
			     $5->code.chars(), $7->code.chars());
		     $$->code = string(buffer);		
				   }

                |  UNI '(' Pred ')'  

                   { 
		     ko_includes.insert( string("FUni"));
		     $$ = new PCode;
		     strlength = strlen("new FUni(") + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FUni(%s)", $3->code.chars());
		     $$->code = string(buffer);	
				   }

                |  INT '(' Pred ')'  

                   { 
		     ko_includes.insert( string("FInt"));
		     $$ = new PCode;
		     strlength = strlen("new FInt(") + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FInt(%s)", $3->code.chars());
		     $$->code = string(buffer);			
				   }

                |  DIF '(' Pred ')'  

                   { 
		     ko_includes.insert( string("FDif"));
		     $$ = new PCode;
		     strlength = strlen("new FDif(") + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FDif(%s)", $3->code.chars());
		     $$->code = string(buffer);		
				   }

                |  SORT '(' Pred ')'

                   {
		     ko_includes.insert(string("FSort"));
		     $$ = new PCode;
		     strlength = strlen("new FSort(") + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FSort(%s)", $3->code.chars());
		     $$->code = string(buffer);	
				   }

                |  PFOLD '(' Func ',' Func ')' 

                   { 
		     ko_includes.insert(string("FPFold"));
		     $$ = new PCode;
		     strlength = strlen("new FPFold(") + strlen($3->code.chars())
		       + strlen(", ") + strlen($5->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FPFold(%s, %s)", $3->code.chars(), $5->code.chars());
		     $$->code = string(buffer);		
				   }

                |  NJOIN '(' Pred ',' Func ',' Func ')'

                   {
		     ko_includes.insert(string("FNJoin"));
		     $$ = new PCode;
		     strlength = strlen("new FNJoin(") + strlen($3->code.chars()) 
		       + strlen(", ") + strlen($5->code.chars()) + strlen(", ") + strlen($7->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FNJoin(%s, %s, %s)", $3->code.chars(), $5->code.chars(), $7->code.chars());
		     $$->code = string(buffer);	
				   }

                |  FOLD '(' Obj ',' Func ',' Func ')'

                   { 
		     ko_includes.insert(string("FFold"));
		     $$ = new PCode;
		     strlength = strlen("new FFold(") + strlen($3->code.chars()) 
		       + strlen(", ") + strlen($5->code.chars()) + strlen(", ") + strlen($7->code.chars())
		       + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FFold(%s, %s, %s)", $3->code.chars(), $5->code.chars(), 
			     $7->code.chars());
		     $$->code = string(buffer);		
		   }

                |  ITERATE '(' Pred ',' Func ')'

                   { 
		     ko_includes.insert( string("FIterate"));
		     $$ = new PCode;
		     strlength = strlen("new FIterate(") + strlen($3->code.chars()) 
		       + strlen(", ") + strlen($5->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FIterate(%s, %s)", $3->code.chars(), $5->code.chars());
		     $$->code = string(buffer);		
				   }

                |  ITER '(' Pred ',' Func ')'   

                   { 
		     ko_includes.insert( string("FIter"));
		     $$ = new PCode;
		     strlength = strlen("new FIter(") + strlen($3->code.chars()) 
		       + strlen(", ") + strlen($5->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FIter(%s, %s)", $3->code.chars(), $5->code.chars());
		     $$->code = string(buffer);	
				   }

                |  PCON '(' Pred ',' Func ')'   

                   { 
		     ko_includes.insert( string("FPCon"));
		     $$ = new PCode;
		     strlength = strlen("new FPCon(") + strlen($3->code.chars()) 
		       + strlen(", ") + strlen($5->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FPCon(%s, %s)", $3->code.chars(), $5->code.chars());
		     $$->code = string(buffer);			
				   }

                |  JOIN '(' Pred ',' Func ')'   

                   { 
		     ko_includes.insert( string("FJoin"));
		     $$ = new PCode;
		     strlength = strlen("new FJoin(") + strlen($3->code.chars()) 
		       + strlen(", ") + strlen($5->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FJoin(%s, %s)", $3->code.chars(), $5->code.chars());
		     $$->code = string(buffer);	
				   }

                |  RSJOIN '(' Pred ',' Func ')'

                   {
		     ko_includes.insert( string("FRSJoin"));
		     $$ = new PCode;
		     strlength = strlen("new FRSJoin(") + strlen($3->code.chars())
		       + strlen(", ") + strlen($5->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FRSJoin(%s, %s)", $3->code.chars(), $5->code.chars());
		     $$->code = string(buffer);	
				   }

                |  LSJOIN '(' Pred ',' Func ')'

                   {
		     ko_includes.insert(string("FLSJoin"));
		     $$ = new PCode;
		     strlength = strlen("new FLSJoin(") + strlen($3->code.chars())
		       + strlen(", ") + strlen($5->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FLSJoin(%s, %s)", $3->code.chars(), $5->code.chars());
		     $$->code = string(buffer);
				   }

                |  UNNEST '(' Func ',' Func ')' 

                   { 
		     ko_includes.insert( string("FUnnest"));
		     $$ = new PCode;
		     strlength = strlen("new FUnnest(") + strlen($3->code.chars()) 
		       + strlen(", ") + strlen($5->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new FUnnest(%s, %s)", $3->code.chars(), $5->code.chars());
		     $$->code = string(buffer);
				   }

                | '(' Func ')'   { $$ = $2;} 

                ;


/********************************************************************************
   Obj
																				 
	Object Rule Expressions.
	Note that object variables (OIDENT) are assumed to be of the form
																				 
	 x, x', x'', ...
	 x1, x2, x3, ...
	 y, y', y'', ...
	 y1, y2, y3, ...
     z, z', z'', ...
	 z1, z2, z3, ...
	 A, A', A'', ...
	 A1, A2, A3, ...
	 B, B', B'', ...
	 B1, B2, B3, ...
	 C, C', C'', ...
	 C1, C2, C3, ...
																				 
********************************************************************************/

Obj             :  PrimO  { $$ = $1; }

                |  FormO  

                   { $$ = $1; }

                |  OIDENT 

                   { 
		     ko_includes.insert(string("OVar"));
		     $$ = new PCode;
		     strlength = strlen("new OVar(\"") + strlen($1) + strlen("\")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new OVar(\"%s\")", $1);
		     $$->code = string(buffer);	
				   }

                |  ODC    

                   {
			ko_includes.insert(string("OVar"));
			$$ = new PCode;
			char count[10];
			sprintf(count, "%d", dontCareCount++);
			strlength = strlen("new OVar(\"_O") + strlen(count) + strlen("\")") + 1;
			buffer = new char[strlength];
			sprintf(buffer, "new OVar(\"_O%s\")", count);
			$$->code = string(buffer);    	
		   }

                ;

PrimO           :  EMPTY      

                   { 
		     ko_includes.insert(string("OSet"));
		     ko_includes.insert(string("ObjList"));
		     $$ = new PCode;
		     strlength = strlen("new OSet(new ObjList())") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new OSet(new ObjList())");
		     $$->code = string(buffer);	
				   }

                |  OID '(' INTCON ')'

                   { 
		     ko_includes.insert(string("OID"));
		     $$ = new PCode;
		     strlength = strlen("new OID(") + strlen($3) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new OID(%s)", $3);
		     $$->code = string(buffer);	
				   }

                |  INTCON     

                   { 
		     ko_includes.insert(string("OInt"));
		     $$ = new PCode;
		     strlength = strlen("new OInt(") + strlen($1) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new OInt(%s)", $1);
		     $$->code = string(buffer);	
				   }

                |  STRINGCON  

                   { 
		     ko_includes.insert( string("OStr"));
		     $$ = new PCode;
		     strlength = strlen("new OStr(\"") + strlen($1) + strlen("\")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new OStr(\"%s\")", $1);
		     $$->code = string(buffer);		
				   }

                |  FLOATCON

                   { 
		     ko_includes.insert( string("OFloat"));
		     $$ = new PCode;
		     strlength = strlen("new OFloat(\"") + strlen($1) + strlen("\")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new OFloat(\"%s\")", $1);
		     $$->code = string(buffer);	
				   }

                |  NIL    

                   { 
		     ko_includes.insert( string("OAny"));
		     $$ = new PCode;
		     strlength = strlen("new OAny") + 1;
		     buffer = new char[strlength];
		     strcpy(buffer, "new OAny");
		     $$->code = string(buffer);	
				   } 

                ;


FormO           :  Func FINVOKE Obj   

                   { 
		     ko_includes.insert( string("OInvoke"));
		     $$ = new PCode;
		     strlength = strlen("new OInvoke(") + strlen($1->code.chars()) + strlen(", ")
		       + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new OInvoke(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);		
				   } 

                |  '[' Obj ',' Obj ']'    

                   { 
		     ko_includes.insert(string("OPair"));
		     $$ = new PCode;
		     strlength = strlen("new OPair(") + strlen($2->code.chars()) + strlen(", ")
		       + strlen($4->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new OPair(%s, %s)", $2->code.chars(), $4->code.chars());
		     $$->code = string(buffer);	
				   }

                |  Obj MULT Obj

                   { 
		     ko_includes.insert( string("OMult"));
		     $$ = new PCode;
		     strlength = strlen("new OMult(") + strlen($1->code.chars()) + strlen(", ")
		       + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new OMult(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);	
				   }

                |  Obj DIV Obj

                   { 
			 ko_includes.insert( string("ODiv"));
			 $$ = new PCode;
			 strlength = strlen("new ODiv(") + strlen($1->code.chars()) + strlen(", ")
			   + strlen($3->code.chars()) + strlen(")") + 1;
			 buffer = new char[strlength];
			 sprintf(buffer, "new ODiv(%s, %s)", $1->code.chars(), $3->code.chars());
			 $$->code = string(buffer);	
				   }

                |  Obj MOD Obj

                   { 
		     ko_includes.insert( string("OMod"));
		     $$ = new PCode;
		     strlength = strlen("new OMod(") + strlen($1->code.chars()) + strlen(", ")
		       + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new OMod(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);	
				   }

                |  Obj MINUS Obj

                   { 
		     ko_includes.insert( string("OMinus"));
		     $$ = new PCode;
		     strlength = strlen("new OMinus(") + strlen($1->code.chars()) + strlen(", ")
		       + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new OMinus(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);		
				   }


                |  Obj PLUS Obj

                   { 
		     ko_includes.insert(string("OPlus"));
		     $$ = new PCode;
		     strlength = strlen("new OPlus(") + strlen($1->code.chars()) + strlen(", ")
		       + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new OPlus(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);		
				   }

                |  IF Bool FTHEN Obj ELSE Obj FI

                    { 
		      ko_includes.insert(string("OIf"));
		      $$ = new PCode;
		      strlength = strlen("new OIf(") + strlen($2->code.chars()) + strlen(", ")
			+ strlen($4->code.chars()) + strlen($6->code.chars()) + strlen(")") + 1;
		      buffer = new char[strlength];
		      sprintf(buffer, "new OIf(%s, %s, %s)", $2->code.chars(), $4->code.chars(), $6->code.chars());
		      $$->code = string(buffer);		 
					}

                |  Obj INTER Obj
  
                    { 
		      ko_includes.insert( string("OInter"));
		      $$ = new PCode;
		      strlength = strlen("new OInter(") + strlen($1->code.chars()) + strlen(", ")
			+ strlen($3->code.chars()) + strlen(")") + 1;
		      buffer = new char[strlength];
		      sprintf(buffer, "new OInter(%s, %s)", $1->code.chars(), $3->code.chars());
		      $$->code = string(buffer);			
					}

                |  Obj ODIF Obj
  
                    { 
		      ko_includes.insert( string("ODif"));
		      $$ = new PCode;
		      strlength = strlen("new ODif(") + strlen($1->code.chars()) + strlen(", ")
			+ strlen($3->code.chars()) + strlen(")") + 1;
		      buffer = new char[strlength];
		      sprintf(buffer, "new ODif(%s, %s)", $1->code.chars(), $3->code.chars());
		      $$->code = string(buffer);	
					}

                |  Obj TIMES Obj
  
                    { 
		      ko_includes.insert( string("OTimes"));
		      $$ = new PCode;
		      strlength = strlen("new OTimes(") + strlen($1->code.chars()) + strlen(", ")
			+ strlen($3->code.chars()) + strlen(")") + 1;
		      buffer = new char[strlength];
		      sprintf(buffer, "new OTimes(%s, %s)", $1->code.chars(), $3->code.chars());
		      $$->code = string(buffer);		
					}

                |  '(' Obj ')'    { $$ = $2; }

                ;



/********************************************************************************
   Bool
																				 
	Bool Rule Expressions.
																				 
	Note that bool variables (BIDENT) are assumed to be of the form
																				 
	 b, b', b'', ...
	 b1, b2, b3, ...
																				 
********************************************************************************/

Bool            :  PrimB  { $$ = $1;}

                |  FormB  

                   { $$ = $1;}

                |  BIDENT 

                   { 
		     ko_includes.insert(string("BVar"));
		     $$ = new PCode;
		     strlength = strlen("new BVar(\"") + strlen($1) + strlen("\")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new BVar(\"%s\")", $1);
		     $$->code = string(buffer);	
				   }

                |  BDC    

                   {
				
				   }

                ;


PrimB           :  TRUEv  

                  { 
		    ko_includes.insert(string("BPrimitive"));
		    $$ = new PCode;
		    buffer = new char[strlen("new BPrimitive(T)") + 1];
		    strcpy(buffer, "new BPrimitive(T)");
		    $$->code = string(buffer);			
	  }

                |  FALSEv 

                   { 
		     ko_includes.insert(string("BPrimitive"));
		     $$ = new PCode;
		     buffer = new char[strlen("new BPrimitive(F)") + 1];
		     strcpy(buffer, "new BPrimitive(F)");
		     $$->code = string(buffer);		
				   }
 
                ;

FormB           :  Pred PINVOKE Obj 

                   { 
		     ko_includes.insert( string("BInvoke"));
		     $$ = new PCode;
		     strlength = strlen("new BInvoke(") + strlen($1->code.chars()) + strlen(", ")
		       + strlen($3->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new BInvoke(%s, %s)", $1->code.chars(), $3->code.chars());
		     $$->code = string(buffer);	
				   }

                |  Obj EQUAL Obj 
                    { 
		      ko_includes.insert( string("BEqu"));
		      $$ = new PCode;
		      strlength = strlen("new BEqu(") + strlen($1->code.chars()) + strlen(", ")
			+ strlen($3->code.chars()) + strlen(")") + 1;
		      buffer = new char[strlength];
		      sprintf(buffer, "new BEqu(%s, %s)", $1->code.chars(), $3->code.chars());
		      $$->code = string(buffer);	

					}
                |  Obj NE Obj
                    { 
		      ko_includes.insert( string("BNotEqual"));
		      $$ = new PCode;
		      strlength = strlen("new BNotEqual(") + strlen($1->code.chars()) + strlen(", ")
			+ strlen($3->code.chars()) + strlen(")") + 1;
		      buffer = new char[strlength];
		      sprintf(buffer, "new BNotEqual(%s, %s)", $1->code.chars(), $3->code.chars());
		      $$->code = string(buffer);	
					}
                |  Obj LESS Obj
                    { 
		      ko_includes.insert(string("BLT"));
		      $$ = new PCode;
		      strlength = strlen("new BLT(") + strlen($1->code.chars()) + strlen(", ")
			+ strlen($3->code.chars()) + strlen(")") + 1;
		      buffer = new char[strlength];
		      sprintf(buffer, "new BLT(%s, %s)", $1->code.chars(), $3->code.chars());
		      $$->code = string(buffer);	
					}
                |  Obj LANGLE Obj
                    { 
		      ko_includes.insert(string("BLT"));
		      $$ = new PCode;
		      strlength = strlen("new BLT(") + strlen($1->code.chars()) + strlen(", ")
			+ strlen($3->code.chars()) + strlen(")") + 1;
		      buffer = new char[strlength];
		      sprintf(buffer, "new BLT(%s, %s)", $1->code.chars(), $3->code.chars());
		      $$->code = string(buffer);		
					}
                |  Obj LE Obj 
                    { 
		      ko_includes.insert( string("BLE"));
		      $$ = new PCode;
		      strlength = strlen("new BLE(") + strlen($1->code.chars()) + strlen(", ")
			+ strlen($3->code.chars()) + strlen(")") + 1;
		      buffer = new char[strlength];
		      sprintf(buffer, "new BLE(%s, %s)", $1->code.chars(), $3->code.chars());
		      $$->code = string(buffer);		 
					}
                |  Obj GREATER Obj 
                    {
		      ko_includes.insert( string("BGT"));
		      $$ = new PCode;
		      strlength = strlen("new BGT(") + strlen($1->code.chars()) + strlen(", ")
			+ strlen($3->code.chars()) + strlen(")") + 1;
		      buffer = new char[strlength];
		      sprintf(buffer, "new BGT(%s, %s)", $1->code.chars(), $3->code.chars());
		      $$->code = string(buffer);	

					}
                |  Obj RANGLE Obj
                    { 
			 ko_includes.insert( string("BGT"));
			 $$ = new PCode;
			 strlength = strlen("new BGT(") + strlen($1->code.chars()) + strlen(", ")
			   + strlen($3->code.chars()) + strlen(")") + 1;
			 buffer = new char[strlength];
			 sprintf(buffer, "new BGT(%s, %s)", $1->code.chars(), $3->code.chars());
			 $$->code = string(buffer);	
		    }
                |  Obj GE Obj
                    { 
		      ko_includes.insert( string("BGE"));
		      $$ = new PCode;
		      strlength = strlen("new BGE(") + strlen($1->code.chars()) + strlen(", ")
			+ strlen($3->code.chars()) + strlen(")") + 1;
		      buffer = new char[strlength];
		      sprintf(buffer, "new BGE(%s, %s)", $1->code.chars(), $3->code.chars());
		      $$->code = string(buffer);	
					}
                |  Bool BAND Bool
                    { 
		      ko_includes.insert( string("BEqu"));
		      $$ = new PCode;
		      strlength = strlen("new BEqu(") + strlen($1->code.chars()) + strlen(", ")
			+ strlen($3->code.chars()) + strlen(")") + 1;
		      buffer = new char[strlength];
		      sprintf(buffer, "new BEqu(%s, %s)", $1->code.chars(), $3->code.chars());
		      $$->code = string(buffer);		
					}
                |  Bool BOR Bool
                    { 
		      ko_includes.insert(string("BOr"));
		      $$ = new PCode;
		      strlength = strlen("new BOr(") + strlen($1->code.chars()) + strlen(", ")
			+ strlen($3->code.chars()) + strlen(")") + 1;
		      buffer = new char[strlength];
		      sprintf(buffer, "new BOr(%s, %s)", $1->code.chars(), $3->code.chars());
		      $$->code = string(buffer);	
					}
                |  NOT '(' Bool ')'
                    { 
					
			 ko_includes.insert(string("BNot"));
			 $$ = new PCode;
			 strlength = strlen("new BNot(") + strlen($3->code.chars()) + strlen(")") + 1;
			 buffer = new char[strlength];
			 sprintf(buffer, "new BNot(%s)", $3->code.chars());
			 $$->code = string(buffer);		}

                | '(' Bool ')'     

                   { $$ = $2;} 

                ;

/********************************************************************************
	Type
																				 
	Type Rule Expressions.
																				 
	Note that type variables (TIDENT) are assumed to be of the form
	 t, t', t'', ...
	 t1, t2, t3, ...
																				 
********************************************************************************/

Type            :  TIDENT  

                   { 
			ty_includes.insert( string("TVar"));
			$$ = new PCode;
			
			strlength = strlen("new TVar(\"") + strlen($1) + strlen("\")") + 1;
			buffer = new char[strlength];
			sprintf(buffer, "new TVar(\"%s\")", $1);
			$$->code = string(buffer);		
				   }

                |  TDC    

                   {
		     ty_includes.insert(string("TDC"));
		     $$ = new PCode;
					  
		     strlength = strlen("new TDC(") + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new TDC()");
		     $$->code = string(buffer);       
				   }

                |  DPO Type DPC

                   { 
		     ty_includes.insert( string("TBag"));
		     $$ = new PCode;
					  
		     strlength = strlen("new TBag(") + strlen($2->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new TBag(%s)", $2->code.chars());
		     $$->code = string(buffer); 	
                   }

                |  '{' Type '}'

                   { 
		     ty_includes.insert( string("TSet"));
		     $$ = new PCode;
					  
		     strlength = strlen("new TSet(") + strlen($2->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new TSet(%s)", $2->code.chars());
		     $$->code = string(buffer);  
                   }              

                |  '[' Type ',' Type ']'

                   { 
		     ty_includes.insert( string("TPair"));
		     $$ = new PCode;
					  
		     strlength = strlen("new TPair(,") + strlen($2->code.chars()) + strlen($4->code.chars()) + strlen(")") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new TPair(%s,%s)", $2->code.chars(), $4->code.chars());
		     $$->code = string(buffer); 		
                   }              

                |  INT

                   { 
		     ty_includes.insert( string("TInt"));
		     $$ = new PCode;
					  
		     strlength = strlen("new TInt()") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new TInt()");
		     $$->code = string(buffer);    		
                   }              

                |  KFLOAT

                   { 
		     ty_includes.insert( string("TFlt"));
		     $$ = new PCode;
		     strlength = strlen("new TFlt()") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new TFlt()");
		     $$->code = string(buffer);   
				
                   }              

                |  KSTRING

                   { 
		     ty_includes.insert(string("TStr"));
		     $$ = new PCode;
		     strlength = strlen("new TStr()") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new TStr()");
		     $$->code = string(buffer);		
                   }         

                |  KCHAR

                   { 
		     ty_includes.insert(string("TChar"));
		     $$ = new PCode;
		     strlength = strlen("new TChar()") + 1;
		     buffer = new char[strlength];
		     sprintf(buffer, "new TChar()");
		     $$->code = string(buffer);   		
                   }        
                ;

%%

int CKerror (char *s)  {
  
  cerr << endl <<  s << " found at line: " << lineno << " (" << CKtext << ")" << " of COKO Property Block.\n\n";
  return 0;
}


