//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Marina Zlatkina
//  FILE: SchemaProperty.C
//  PROJ: COKO
//  DATE: 8/2001                                                        
//
// SYNOPSIS
//  Class that represents a fact. The isTrue method determines if this fact
//  is stored in the Schema file.
//  
//////////////////////////////////////////////////////////////////////////////
#include "SchemaProperty.H"
#include "CGlobals.H"
#include "NonTypeList.H"
#include "KOBVar.H"
#include "KOFVar.H"
#include "KOPVar.H"
#include "KOOVar.H"
#include <fstream>
#include <regex.h>

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: SchemaProperty
// Parameters:	(string, NonTypeArgs)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////
SchemaProperty::SchemaProperty(string s, NonTypeArgs *nt):key(s), tree(nt){}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: isTrue
// Parameters:	(CState *, NonTypeArgs *)
//
// Determines if Schema.txt contains the fact we need to verify.
//         								    
//////////////////////////////////////////////////////////////////////////////
NonTypeArgs * SchemaProperty::isTrue(CState *s, NonTypeArgs *a){
  
  string current;
  regex_t myRegex;
  string temp;

  // substitute all variables in the arguments with their corresponding values
  NonTypeArgs *args = tree->build(s);
 
  // set the search string to key (the name of the fact) and the
  // regular expression representation of the arguments
  string searchString = key + string(args->representation());
 
  // open Schema.txt
  ifstream file( SCHEMA_LOCATION, ios::in );
  
  if (!file)
  {
     std::cout << "File could not be opened" << std::endl;
        
     return new NonTypeArgs(new NonTypeList(),0);
  }
  
  // compile a regular expression 
  regcomp(&myRegex, searchString.c_str(), REG_EXTENDED|REG_ICASE);

  // create an array of regmatch_t objects to contain match information
  // make it 1 more then the number of possible matches
  regmatch_t matches[myRegex.re_nsub+1];
  size_t size = myRegex.re_nsub+1;

  if (DEBUG_PROP)
    std::cerr<<searchString<<endl;

  // while there is something to read from the file
  while ( file ){ 
    // store the next line in current
    getline(file, current);
    
    // if the regular expression matched the line from the Schema file
    if ( regexec( &myRegex, current.c_str(), size, matches,0) == 0 ){
      
      regfree(&myRegex);
      int cur = 1;
      
      NonTypeList *list = args->theList();
      list->initIterator();
      NonTypeList *newList = new NonTypeList();
      NonType *curMember;

      // for every NonType in args (the argument list with as many variables as possible
      // replaced with values)
      while (curMember=list->getValue()){
	
	NameUnion *tmp = curMember->Myname();
	
	// if the argument is a boolean variable, insert into the list another boolean variable 
	// with the name of the form: var-name:var-value, where var-name is just the name of
	// the variable that was never replaced with a value (the name from the original COKO
	// transformation), and var-value is the value the regular expression matched for us.
	if ((tmp->tag==BNAME) && (tmp->bn == BOOL_VAR)){
	  temp = curMember->Name();
	  temp.append( ":" + current.substr(matches[cur].rm_so, matches[cur].rm_eo - matches[cur].rm_so));
	  newList->insert( new BVar((char*)temp.c_str() ) );
	    cur++;
	  }

	// if the argument is a function variable, insert into the list another function variable
	// as per above explanation
	if ((tmp->tag==FNAME) && (tmp->fn == FUN_VAR)){
	  temp = curMember->Name();
	  temp.append(":"+current.substr(matches[cur].rm_so, matches[cur].rm_eo - matches[cur].rm_so));
	  newList->insert( new FVar((char*)temp.c_str() ) );
	  cur++;
	}

	// if the argument is a predicate variable, insert into the list another predicate variable
	// as per above explanation
	if ((tmp->tag==PNAME) && (tmp->pn == PRED_VAR)){
	  temp = curMember->Name();
	  temp.append(":"+current.substr(matches[cur].rm_so, matches[cur].rm_eo - matches[cur].rm_so));
	  newList->insert( new PVar((char*)temp.c_str() ) );
	  cur++;
	}

	// if the argument is a predicate variable, insert into the list another predicate variable
	// as per above explanation
	if ((tmp->tag==ONAME) && (tmp->on == OBJ_VAR)){
	  temp = curMember->Name();
	  temp.append( ":" + current.substr(matches[cur].rm_so, matches[cur].rm_eo - matches[cur].rm_so));
	  newList->insert( new OVar((char*)temp.c_str() ) );
	  cur++;
	}
      }
      
      // create new NonTypeArgs object containing the list with the variables we just made
      NonTypeArgs * newArgs =  new NonTypeArgs(newList,cur-1);

      // set the success value to true
      newArgs->SetSuccess(true);
      
      return newArgs;
    }       
  }


  regfree(&myRegex);

  // the string was not found, so return a blank NonTypeArgs object (success defaults to false)
  return new NonTypeArgs(new NonTypeList(), 0);
}
