//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Daniel Abadi
//  FILE: HistoryList.H
//  PROJ: COKO DEBBUGGER
//  DATE: WINTER 2001                                                    
//
// SYNOPSIS
//  Declarations for History Stack.
//  
//////////////////////////////////////////////////////////////////////////////


#ifndef _HISTORYLIST_H_
#define _HISTORYLIST_H_

#include <stdlib.h>
#include "KONonType.H"
#include "PCIEnvStack.H"
#include <iostream.h>

class HistoryNode
{
public:
  NonType * _value;
  int _location;
  int _which;
  CIEnvStack* _environmentStack;
  char* _variableName;
  HistoryNode * _next;
  NonType* _pattern;

  HistoryNode(NonType* value, int location, int which, char* v_name, CIEnvStack* eStack, NonType* pattern);
  // Constructor

  ~HistoryNode();
  // Destructor

  void display();
  //for debugging purposes only

};

class HistoryList
{
private:
  HistoryNode * _iterator;
  HistoryNode * _head;
  HistoryNode * _tail;

public:

  HistoryList();
  // Constructor
  
  ~HistoryList();
  // Destructor

  void push(NonType* a, int num, int which, char* v_name, CIEnvStack* eStack, NonType* pattern);
  //  Push a new node with _value = a and _num = num to the stack

  int pop();
  // Pop off a front node from the stack
  // Return success value of the pop.

  NonType* topVal();
  // Returns _value of the top node.

  int topLocation();
  // Returns _location of the top node.

  int topWhich();
  // Returns _which of the top node.

  int empty();
  // Is the list empty?

  void initIterator();
  // Initialize _iterator
  // This method is used with getValue method

  NonType* iteratorValue();
  // Returns _value of the node that _iterator is pointing to.

  int iteratorLocation();
  // Returns _location of the node that _iterator is pointing to.

  int iteratorWhich();
  // Returns _which of the node that _iterator is pointing to.

  CIEnvStack* iteratorStack();
  // Returns _environmentStack of the node that _iterator is pointing to.

  char* iteratorName();
  // Returns _variableName of the node that _iterator is pointing to.

  NonType* iteratorPattern();
  // Returns _pattern of the node that _iterator is pointing to.

  NonType* getNextValue();
  // Point _iterator to the next node and returns the _value of that node.
  // One can call this methods repeatly to get the _value of all the nodes.
  // However, initIterator has to be called before the initial call to this 
  // method.  

  void display();
  //for debugging purposes only

};

#endif

