//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Kee-Eung Kim (kek)                                             
//  FILE: PCRuleBlock.H
//  PROJ: COKO
//  DATE: 8/27/96                                                              
//
// SYNOPSIS
//  Class definition for CRuleBlock class.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _PCRULEBLOCK_H_
#define _PCRULEBLOCK_H_

//#include <PCStmt.H>
#include <PCState.H>
//#include <DEDebugController.H>

//////////////////////////////////////////////////////////////////////////////
//
//  CLASS: CRuleBlock
//
// SYNOPSIS
//  This class is an abstract base class for rule blocks. When the COKO parser
//  generates C++ code for a rule block, it will make the rule block a
//  derived class of CRuleBlock.
//
//////////////////////////////////////////////////////////////////////////////

class CStmt;
class DDebugController;
class UsesList;

class CRuleBlock 
{
public:
  
  virtual CState *Exec(CState*);// It does nothing by default;


  //DJA SUMMER 2001:
  //Added representation, get_name, get_stmt, get_num_stmts, construct_stmt, number_self, output_self, is_leaf and ExecDebug methods
  
  virtual CState *ExecDebug(CState*,DDebugController*); 
  // It does nothing by default;
  
  virtual char* get_name();
  //get name of RuleBlock

  virtual void construct_stmt();
  //create the COKO statement that Exec will be called on.

  virtual CStmt* get_stmt();
  //get the COKO statement that Exec will be called on.
  
  virtual int number_self(int);
  //assigns an integer to the current node for use in the
  //debugger for the breakOptions command
  
  virtual void output_self(DDebugController*);
  //output key_number and rep_string for debugger
  //for breakOptions command

  virtual char* representation (); 
  //returns the string representation of the COKO subtree
  //beginning at this object

  virtual char* rep_with_wrapper (char *); 
  //Takes the maincontent of the ruleblcok and adds USES section, BEGIN
  //END, etc. and returns string representation of COKO tree stored in this 
  //RuleBlock

  virtual char* representation_with_debugging_info (DDebugController *db); 
  //returns the string representation of this COKO RuleBlock, but
  //at the same time updates an array in db that locates sub-strings for
  //each key number

  virtual int get_num_stmts();
  //returns value stored in num_stmts variable

protected:
  
  int constructed_stmt;
  //Added DJA SUMMER 2001
  //boolean signifying whether the construct_stmt method has been called yet

  int numbered_self;
  //Added DJA SUMMER 2001
  //boolean signifying whether the number_self method has been called yet
  
  int num_stmts;
  //Added DJA SUMMER 2001
  //integer storing the total number of statments in the RuleBlock
  //this is unknown until the number_self method is called

  CStmt * _rStmt_;
  //Added DJA SUMMER 2001
  //This is the main statement that the Exec method will execute

  UsesList* _useList;
  //Added DJA SUMMER 2001
  //this is a list of Rules and RuleBlocks used by this transformation
  //used in outputting the content of the USES section of the code

  
};

#endif

