/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOOName.C
// PROJ: COKO Compiler, KOLA Parcer
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOOName.H"

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  OName
// Parameters:  (char* s)
//
//  Constructor.  s is the name of the identifier.
//////////////////////////////////////////////////////////////////////////////

OName::OName(char* s, TObj * t) :
             koObject(t, s)
{
  if (s == NULL) val = NULL;
  else {
    val = new char [strlen(s) + 1];
    sprintf (val, "%s", s);
  }

  db = NULL;
  ostore_obj = NULL;
  database_name = NULL;
  variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  OName
// Parameters:  (os_database *dbase, char* s, char* db_name)
//
//  Constructor.  s is the name of the identifier.
//                dbase is a pointer to an ObjectStore database.
//         		  db_name is the database name.						    
//////////////////////////////////////////////////////////////////////////////

OName::OName(os_database *dbase, char *db_name, char* s, TObj * t) :
             koObject(t, s)
{

  if (s == NULL) val = NULL;
  else {
    val = new char [strlen(s) + 1];
    sprintf (val, "%s", s);
  }

  if (dbase)
	db = dbase;
  else
	db = NULL;

  ostore_obj = NULL;

 if (db_name)
	{
	  database_name = new char [strlen(db_name) +1];
	  sprintf(database_name, "%s", db_name); 
	}
  else
	database_name = NULL;
 variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OName
// Parameters:	(OSObject* o, os_database *dbase, char* s, char* db_name)
//
//  Constructor.  o is a pointer an OSObject
//         	  dbase is a pointer to an ObjectStore database.    
//         		  db_name is the database name.						    
//////////////////////////////////////////////////////////////////////////////

OName::OName(OSObject *o, os_database *dbase, char* db_name, char *s, TObj * t) :
             koObject(t, s)
{

  if (s == NULL) val = NULL;
  else {
    val = new char [strlen(s) + 1];
    sprintf (val, "%s", s);
  }


  if (dbase)
	db = dbase;
  else
	db = NULL;

  if (o)
	ostore_obj = o;
  else
	ostore_obj = NULL;

 if (db_name)
	{
	  database_name = new char [strlen(db_name) +1];
	  sprintf(database_name, "%s", db_name); 
	}
  else
	database_name = NULL;
 variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OName
// Parameters:	(char* s, char* db_name)							  
//
//  Constructor.  s is the name of the identifier.
//  db_name is the name of the Objectstore database.
//         								    
//////////////////////////////////////////////////////////////////////////////

OName::OName (char* s, char* db_name, TObj * t) : koObject (t, s)
{
  if (s == NULL) val = NULL;
  else {
    val = new char [strlen(s) + 1];
    sprintf (val, "%s", s);
  }

  if (db_name)
	{
          char* tmp_db_name;
          tmp_db_name = new char [strlen(db_name) + 4];
          sprintf(tmp_db_name, "%s.db", db_name);
#ifdef _OSTORE_
          db = os_database::open(tmp_db_name);
#endif
          database_name = new char [strlen(db_name) +1];
          sprintf(database_name, "%s", db_name);
          if (tmp_db_name) { delete [] tmp_db_name; tmp_db_name = NULL;}
	}
  else
	{
	  database_name = NULL;
	  db = NULL;
	}
  
  ostore_obj = NULL;
  variableNum = 0;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OName
// Parameters:	(OName& other)							  
//
//  Copy Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

OName::OName (const OName& other) : koObject ((koObject &) other)
{

  if (other.val == NULL) val = NULL;
  else {
    val = new char [strlen(other.val) + 1];
    sprintf (val, "%s", other.val);
  }

  if (other.db)
	db = other.db;
  if (other.ostore_obj)
	ostore_obj = other.ostore_obj;
  if (other.database_name)
	{
	  database_name = new char [strlen(other.database_name) +1];
	  sprintf(database_name, "%s", other.database_name);
	} 	
  else
	database_name = NULL;
  variableNum = other.getVariableNum();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~OName
// Parameters:	()							  
//
//  Destructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

OName::~OName ()
{
  if (val) {delete [] val; val = NULL;}
  db = NULL;
  ostore_obj = NULL;
  if (database_name)
	{
	  delete [] database_name; database_name = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  return copy of this as koObject*
//         								    
//////////////////////////////////////////////////////////////////////////////

koObject* OName::Copy () const
{
  OName* temp = new OName (*this);
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  deepeq
// Parameters:  ()
//
// Value-based equality.
// this is computed recursively.
// Return true if two trees have identical structures, otherwise return false.
// No unification occurs here.
//
//////////////////////////////////////////////////////////////////////////////
 
int OName::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == ONAME ) && (name_->on == OBJ_NAME))
  {
    delete name_;
    if (!strcmp(((OName*)other)->Name(),val))
      return T;
    else
      return F; 
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* OName::Myname()
{
	NameUnion* a = new NameUnion(ONAME, OBJ_NAME);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//  Return identifier result.
//
/////////////////////////////////////////////////////////////////////////////

// OResult *OName::reduce ()
// {
//   return new OResID ((char *) Name());
// }


///////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  Return "OPRIM NAME: " + name
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OName::representation ()
{
if (database_name)
{
  char *res = new char [strlen (Name()) + strlen ("Obj(,)") + strlen(database_name) + 1]; 
  sprintf (res, "Obj(%s,%s)", Name(),database_name);
  return res;
}
else
{
  char *res = new char [strlen (Name()) + strlen ("Obj()") + 1];
  sprintf (res, "Obj(%s)",Name());
  return res;
}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* OName::transform(CState* s)
{
  OName* temp = new OName(val, database_name,Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here.
//
// For example, foo: g o g --> g o g is a legal rule.
//                  -------   -------
//                    |         |_\
//                    |_\         / building part
//                      / matching part
//
// Assume that we apply this rule to a kola tree of the form "ID o ID."
// When unification occurs in a matching part, first we store first g and ID pair
// into the env stack. Then the next time we face another g (second one),
// we search the stack for the matching pair with variable name g and simply check
// if the matching pair in the stack has same structure as the matching pair we have.
// In this case, second pair is (g, ID) as well and the rule gets fired successfully.
//
// Now assume that we apply the same rule to a kola tree of the form "ID o (ID o ID)."
// In this case, the rule fails since the second pair is (g, ID o ID),
// while the first pair is (g, ID).
//
// Note that even in a case of failure, the rule itself is legal.
//
// "Duplication of a variable is legal in a rule section."
//
//////////////////////////////////////////////////////////////////////////////

int OName::compares(NonType* nt, CState* a)
{
  NameUnion* tmp_name = nt->Myname();
 
  if ((tmp_name->tag == ONAME) && (tmp_name->on == OBJ_NAME) && (!strcmp(((OName*)nt)->Name(),val)))
	{
	  delete tmp_name;
	  return T;
	}
  else
	{
	  delete tmp_name;
	  return F;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void OName::replaceChild(NonType* child, int which_child)
{
}

#ifdef _OSTORE_
//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: result
//  Parameters:
//
// Returns the OSObject equivalent of itself.
//
//////////////////////////////////////////////////////////////////////////////

OSObject * OName::result()
{
  if (ostore_obj != NULL)
  {
    return(ostore_obj);
  }
  else
  {
    return (new Source(Name(), db));
  }
}

#endif // _OSTORE_

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* OName::prolog ()
{
  char *res = new char [strlen (Name()) + strlen ("oname(k)") + 1];
  sprintf (res, "oname(k%s)", Name());
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref OName::sp_prolog ()
{
  char* text = new char [strlen (Name()) + 2];
  sprintf (text, "k%s", Name());
  
  SP_term_ref nam = SP_new_term_ref();
  SP_term_ref res = SP_new_term_ref();
  SP_put_string(nam,text);
  
  SP_cons_functor(res, SP_atom_from_string("oname"),1,nam);
  
  if (text) { delete text; text = NULL; }
  
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	()
//
//  Return the SQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OName::sql ()
{
  char *res = new char [strlen (val) + 1];
  sprintf (res, val);
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	()
//
//  Return the OQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OName::oql ()
{
  char *res = new char [strlen (val) + 1];
  sprintf (res, val);
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Val
// Parameters:	()
//
// Selector - return a copy of val
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OName::Val ()
{
  char *res = new char [strlen (val) + 1];
  sprintf (res, "%s", val);
  return res;
}
