/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOOSet.C
// PROJ: COKO Compiler, KOLA Parcer
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOOSet.H"
//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OSet
// Parameters:	(ObjList* ol)							  
//
//  Constructor.  ol is the list of objects
//  that denote the members of the set.
//         								    
//////////////////////////////////////////////////////////////////////////////

OSet::OSet (ObjList* ol, TObj * t) : koObject (t)
{
    mems = ol;
    namestring = representation();
    variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OSet
// Parameters:	(ObjList* ol)							  
//
//  Copy Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

OSet::OSet (const OSet& other) : koObject ((koObject &) other)
{
    mems = new ObjList (*(other . Members ()));
    namestring = representation();
    variableNum = other.getVariableNum();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~OSet
// Parameters:	()
//
//  Constructor.  ol is the list of objects
//  that denote the members of the set.
//         								    
//////////////////////////////////////////////////////////////////////////////

OSet::~OSet ()
{
    delete mems;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  return copy of this as koObject*
//         								    
//////////////////////////////////////////////////////////////////////////////

koObject* OSet::Copy () const
{
  OSet* temp = new OSet (*this);
  temp->setVariableNum(variableNum);
  return temp;
}


//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  deepeq
// Parameters:  ()
//
// Value-based equality.
// this is computed recursively
//
//////////////////////////////////////////////////////////////////////////////
 
int OSet::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == ONAME ) && (name_->on == OBJ_SET))
  {
    delete name_;
    return mems->listeq(((OSet*)other)->Members());
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* OSet::Myname()
{
	NameUnion* a = new NameUnion(ONAME, OBJ_SET);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//  return 
//    new OResLISTEDSET (ol -> reduce ());
//
/////////////////////////////////////////////////////////////////////////////

// OResult* OSet::reduce ()
// {
//     return new OResLISTEDSET (mems -> reduce ());
// }


// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  Return "OPRIM: SET " + mems.representation ()
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OSet::representation ()
{
    char *mr = mems -> representation ();
    char *res = new char [strlen (mr) + strlen ("{}") + 1];
    sprintf (res, "{%s}", mr);
    delete [] mr;
    return res;
}    

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* OSet::transform(CState* s)
{
  OLink* tmpLink;
  tmpLink = mems->First();

  // first creat empty ObjList.
  ObjList* result_list = new ObjList();

  // when original list is empty, return empty list.
  // else copy (transform) every member in original list into new list.
  while (tmpLink != NULL)
	{
	  result_list->insert((koObject*)((tmpLink->Obj())->transform(s)));
	  tmpLink = tmpLink->Next();
	}
  OSet* temp = new OSet(result_list,Type());
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here. 
//
//////////////////////////////////////////////////////////////////////////////

int OSet::compares(NonType* nt, CState* a)
{
  NameUnion* tmp_name = nt->Myname();
 
  if ((tmp_name->tag == ONAME) && (tmp_name->on == OBJ_SET))
	{
	  delete tmp_name;
	  return T;
	}
  else 
	{
	  delete tmp_name;
	  return F;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void OSet::replaceChild(NonType* child, int which_child)
{
}
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* OSet::prolog ()
{
  char *mr = mems -> prolog ();
  char *res = new char [strlen (mr) + strlen ("oset()") + 1];
  sprintf (res, "oset(%s)", mr);
  if (mr) { delete [] mr; mr = NULL; }
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref OSet::sp_prolog ()
{
  SP_term_ref or = mems -> sp_prolog ();
  SP_term_ref res = SP_new_term_ref();
  SP_cons_functor(res, SP_atom_from_string("oset"),1,or);
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	()
//
//  Return the SQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OSet::sql ()
{
  char *arg1 = mems -> sql ();

  if (strncmp (arg1, "ERROR", 5) == 0) {
	return arg1;
  } else {
	char *res = new char [strlen ("set ()") + strlen (arg1) + 1];
	sprintf (res, "set (%s)", arg1);
	delete [] arg1;
	return res;
  }

}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	()
//
//  Return the OQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OSet::oql ()
{
  char *arg1 = mems -> oql ();

  if (strncmp (arg1, "ERROR", 5) == 0) {
	return arg1;
  } else {
	char *res = new char [strlen ("set ()") + strlen (arg1) + 1];
	sprintf (res, "set (%s)", arg1);
	delete [] arg1;
	return res;
  }

}



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  obj
// Parameters:	()
//
// Symbolic simplification of this.  Eventually,
// this method will replace "object" presently
// only defined with _OSTORE_ flag set
//  
//////////////////////////////////////////////////////////////////////////////

koObject* OSet::obj ()
{
  return new OSet (mems -> objlist ());
}

