/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Joon S. Lee (jl)					 	    
//  FILE: KOPExists.H 
//  PROJ: COKO, KOLA
//  DATE: 5/97		                                                    
//
// SYNOPSIS
//  Class of existentially quantifying predicate former.
//  Given predicate p, we have
//  
//    exists (p) ? A <=> \E a (a IN A /\ p ? a)
//    
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_PCEXISTS_HEADER
#define KOLA_PCEXISTS_HEADER 

#include "KGlobals.H"
#include "KOPredicate.H"
#include "KOObject.H"
#include "KONonType.H"
#include "KOBPrimitive.H"
#include "KOBInvoke.H"

#ifdef _OSTORE_
#include "SODA_GLOBALS.H"
#include "Iterator.H"
#include <dlfcn.h>
#endif // _OSTORE_

#include "PCState.H"
#include "KOPVar.H"

#include <string.h>

typedef NonType* (*callOptPtr)(NonType*);

class PExists : public Predicate
{
    
 public:
  PExists (Predicate* p, TPred * t = NULL);
  //  Constructor.  p is predicate over which to existentially
  //  quantify.

  int compares(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 

  int comparesSTM(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This match method is used for matching statement.
  // No duplicated variable entry is allowed.
  
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.
 
  void setDEPnInvDEP(char*, CState*);
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*);
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  NonType* transformDeep(CState* s, CIEnvStackDataType* startingPt);
  // difference between transform and 
  // transformDeep is that the former uses
  // IFind method and later uses IFindDeep 
  // method. 

  void replaceChild(NonType* child, int which_child);
  // Replace a child of given tree.
  
  PExists (const PExists& other);
  //  Copy Constructor. 
  
  ~PExists ();
  //  Destructor. 

  Predicate *Copy () const;
  // Virtual Copy Constructor
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int numChildren() const { return 1; }
  //  Number of arguments.

  Predicate* One () const {return one;}
  //  What is the Predicate that is the first argument?
    
  //  BResult* reduce (OResult* a);
  //  v = OResVar ();
  //  Return new BResFA 
  //   (v,
  //    new BResCOM
  //      (new BResINF (v, "IN", a -> reduce ()),
  //       "/\",
  //       p -> reduce (v)
  //      )
  
  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  //  Return "PFO: EXISTS " + one -> representation ()

  const char * Name() const { return namestring; }
  // Return the text representation of the name.
    
  void SetOne(NonType* a) { delete one; one = (Predicate*)a; }
  // replace existing child to new one.
 
  void initCurrChild() {currChild = 0;}
  // set (reset) currChild to 0.
  // This needs to be called before using nextChild() for first time.

  NonType* nextChild();
  // returns next child. (next Child = current Child + 1) 
  //If current child is the last child return NULL.

  char *oql (koObject *o);
  // Return the OQL string equivalent of invoking this on o

  char *sql (koObject *o);
  // Return the SQL string equivalent of invoking this on o

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koBool* invoke(koObject* x);

  OSBool* exec(OSObject* x);
  //  Implements the existentially quantifying predicate former.
#endif // _OSTORE_

protected:
  Predicate* one;
  //  What is the Predicate that is the 1st argument?

  int currChild;
};


# endif
