/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack(mfc)
//        Xiaoyu Wang(wangxy)
//  FILE: KOPMatch.C 
//  PROJ: COKO, KOLA
//  DATE: 9/00		                                                    
//
// SYNOPSIS
//  The "match" predicate primitive for
//  tagexp.  
//    
//////////////////////////////////////////////////////////////////////////////

#include "KOPMatch.H"
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  PMatch
// Parameters:  (TagExp* te)
// 
// Constructor. te is tagexp argument.
//
//////////////////////////////////////////////////////////////////////////////
  
PMatch::PMatch (TagExp *te, TPred * t) : Predicate(t, PRED_MATCH, "MATCH")
{
     one = te;
     variableNum = 0;
}


/////////////////////////////////////////////////////////////////////////////
//
// Function Name:  PMatch
// Parameters:  (PMatch& other)
//
// Copy Constructor.
//
/////////////////////////////////////////////////////////////////////////////

PMatch::PMatch(const PMatch& other) : Predicate((Predicate &) other)
{
    one = other.One() ->Copy();
    variableNum = other.getVariableNum();
}

/////////////////////////////////////////////////////////////////////////////
//
// Function Name: ~PMatch
// Parameters: ()
//
// Destructor
//
/////////////////////////////////////////////////////////////////////////////

PMatch::~PMatch ()
{
    delete one;
}	
  
/////////////////////////////////////////////////////////////////////////////
//
// Function Name: Copy ()
// Parameters: ()
//
// Return copy of *this as Predicate
//
////////////////////////////////////////////////////////////////////////////

Predicate*  PMatch::Copy () const
{
  PMatch* temp = new PMatch (*this);
  temp->setVariableNum(variableNum);
  return temp;
}

////////////////////////////////////////////////////////////////////////////
//
// Function Name: deepeq
// Parameters: (NonType *)
//
// does nothing
//
////////////////////////////////////////////////////////////////////////////

int PMatch::deepeq(NonType * other)
{
  return T;
}

////////////////////////////////////////////////////////////////////////////
//
// Function Name:  Myname
// Parameters: ()
//
// does nothing
//
///////////////////////////////////////////////////////////////////////////

NameUnion* PMatch::Myname()
{
  return NULL;
}

///////////////////////////////////////////////////////////////////////////
//
// Function Name: nextChild
// Parameters: ()
//
// Returns "currChild++"th child.
// When currChild is equal to the number of children
// (when currChild is the last child), it returns NULL.
//
///////////////////////////////////////////////////////////////////////////

NonType* PMatch::nextChild()
{
  if(currChild == 0)
    {
      currChild++;
      return one;
    }
  else return NULL;
}
 
///////////////////////////////////////////////////////////////////////////
//
// Function Name:  representation
// Parameters: ()
//
// "PFO: MATCH" + one -> representation ()
//
///////////////////////////////////////////////////////////////////////////

char * PMatch::representation ()
{
    char *pr = one ->representation ();
    char *res = new char[strlen(pr)+strlen("MATCH ()")+1];
    sprintf(res, "MATCH (%s)", pr);
    delete [] pr;
    return res;
}

///////////////////////////////////////////////////////////////////////////
//
// Function Name: transform
// Parameters: (CState*)
//
// Return coresponding NonType object.
// CState is only used in VAR classes.
// No other classes need CState.
// However, CState is kept for making transform methods uniformly.
//
///////////////////////////////////////////////////////////////////////////

NonType* PMatch::transform(CState *s)
{
  TagExp * first = (TagExp*) One() -> transform(s);
  PMatch* temp = new PMatch(first, Type());
  temp->setVariableNum(variableNum);
  return temp;
}


///////////////////////////////////////////////////////////////////////////
//
// Function Name: transformDeep
// Parameters: (CState *)
//
// Do not know what to do... 
//
///////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////
// 
// Function Name: compares
// Parameters: (NonType *nt, CState* a)
//
// 
//////////////////////////////////////////////////////////////////////////

int PMatch::compares(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion * tmp_name = nt ->Myname();
  
  if((tmp_name->tag == PNAME) && (tmp_name ->pn == PRED_MATCH))
    {
      delete tmp_name;
      return T;
    }
  else
    {
      delete tmp_name;
      return F;
    }
}


//////////////////////////////////////////////////////////////////////////////
//
// Function Name: compareSTM
// Parameter: (NonType*, CState*)
//
// Checks for match between two NonType tree (query tree and pattern tree).
// Store a variable and CIdent pair onto the stack.
// This match method is used for matching statement.
// No duplicated variable entry is allowed.
//
//////////////////////////////////////////////////////////////////////////////

int PMatch::compareSTM(NonType* nt, CState* a)
{
     return T;
}

/////////////////////////////////////////////////////////////////////////
//
// Function Name:  replaceChild
// Parameters:  (NonType*, int)
//
// Replace a child of a given tree.
// which_child specifies which child to replace.
/// For example, when which_child is one, we replace the first child of
// given tree.
//
/////////////////////////////////////////////////////////////////////////

void PMatch::replaceChild(NonType* child, int which_child)
{
  if(which_child == 1)
     setOne(child);
}

#ifdef _OSTORE_
/////////////////////////////////////////////////////////////////////////
//
// Function Name: invoke
// Parameters: (koObject*)
//
// Implements the predicate match former for koObjects.
//
/////////////////////////////////////////////////////////////////////////

koBool* PMatch::invoke( koObject* x )
{
  return new BInvoke(this, x);
}

/////////////////////////////////////////////////////////////////////////
//
// Function Name: exec
// Parameters:(OSObject*)
//
// Implements the predicate conjunction former of OSObjects.
//
/////////////////////////////////////////////////////////////////////////

OSBool* PMatch::exec(OSObject* x)
{
  if(((OSBPrimitive*)One()->exec(x))->Val())
     return new OSBPrimitive(0);
  else
     return new OSBPrimitive(1);
}
#endif // _OSTORE_

////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters: ()
//
// return string representation in prolog term
//
////////////////////////////////////////////////////////////////////////

char* PMatch::prolog()
{
    char *pr = one->prolog();
    char *res = new char [strlen(pr)+strlen("pnegate()")+1];
    sprintf(res, "pnegate(%s)", pr);
    delete [] pr;
    return res;
} 

#ifdef _PROLOG_
///////////////////////////////////////////////////////////////////////
//
// Function Name: sp_prolog
// Parameters: ()
//
// since C/C++ function only have indirect access to Prolog terms
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info
// on this usuage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////
SP_term_ref PMatch::sp_prolog()
{
    SP_term_ref or = one -> sp_prolog();
    SP_term_ref res = SP_new_term_ref();
    SP_cons_functor(res, SP_atom_from_string("pmatch", 1, or);
    return res;
}
#endif //_PROLOG_

#ifdef _OSTORE
//////////////////////////////////////////////////////////////////////
//
// Function Name: invoke	 
// Parameters: (koObject *o)
//
// Symbolic invocation of this on o. Eventually, this method will 
// replace "invoke" presently only defined with _OSTORE_ flag set
//
//////////////////////////////////////////////////////////////////////
koBool* PMatch::invoke(koObject* x)
{
   return new BInvoke(this, x);
}

#endif //_OSTORE

