/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack(mfc) 
//        Xiaoyu Wang(wangxy)
//  FILE: KOPMatch.H 
//  PROJ: COKO, KOLA
//  DATE: 9/00		                                                    
//
// SYNOPSIS
//  The "match" predicate primitive for
//  tagexp.  
//    
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_PMATCH_HEADER
#define KOLA_PMATCH_HEADER 

#include "KGlobals.H"
#include "KOPredicate.H"
#include "KOTagExp.H"
#include "KOOPair.H"
#include "KONameUnion.H"
#include "KOBInvoke.H"

#ifdef _OSTORE_
#include "OSBPrimitive.H"
#endif // _OSTORE_

#include "PCState.H"
#include "KONonType.H"
#include <string.h>

class PMatch : public Predicate
{
public:
  PMatch (TagExp *te, TPred * t = NULL);
  //  Constructor, te is the TagExpression to match.
  
  int compares(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 

  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.

  void replaceChild(NonType* child, int which_child);
  // Replace a child of given tree.

  void setDEPnInvDEP(char*, CState*) {}
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*) {}
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  int compareSTM(NonType* other, CState* s);
  
  PMatch (const PMatch& other);
  //  Copy Constructor.
  
  ~PMatch ();
  //  Destructor.
  
  Predicate *Copy () const;
  // Virtual Copy Constructor.

  int deepeq(NonType * other);
  // Value-based equality
 
  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int numChildren() const { return 1; }
  //  Number of arguments.

  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  //  "PPRIM: MATCH"

  TagExp *One() const { return one; }
  //what's the tagexp that is the first argument

  void setOne(NonType *te){ delete one; one = (TagExp *) te; }
  //replace existing child to new one.

  void initCurrChild() {currChild = 0;}
  //set (reset) currChild to 0.
  // This needs to be called before using nextChild() for first time. 

  NonType* nextChild(); 
  // return next child.(next Child = current Child + 1)
  // If current child is the last child return NULL.

  koBool *inv (TagExp *o);
  // Symbolic invocation of this on o. Eventually, 
  // this method will replace "invoke" presently
  // only defined with _OSTORE_ flag set 

  const char * Name() const { return namestring; }
  // Return the text representation of the name.

  
  char *oql (koObject *o){ return namestring;}
  // Return the OQL string equivalent of invoking this on o
  // ERROR

  char *sql (koObject *o){ return namestring;}
  // Return the SQL string equivalent of invoking this on o
  // ERROR

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koBool* invoke(koObject* x);
  //  Implements the match predicate former for koObjects.

  OSBool* exec(OSObject* x);
  //  Implements the match predicate former for OSObjects.
#endif // _OSTORE_

protected:
  TagExp* one;
  // what's the tagexp that is the 1st argument?
  
  int currChild;
};

#endif
