
/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack(mfc) 
//        Xiaoyu Wang(wangxy) 
//  FILE: KOTagExp.H 
//  PROJ: COKO, KOLA
//  DATE: 9/00		                                                    
//
// SYNOPSIS
//  The class of containable tagexpressions.  (Includes
//  Tilde, Dollar, Bang, DoubleBang, Star and Question). 
//  typed objects.  This class and its subclasses
//  construct object that are used in KOLA trees.
//  These should be distinguished from
//  OResults which are used in expression trees.
//  KOLA trees are used for optimization.  Expression
//  trees are used to describe the result of a query denoted 
//  by a KOLA tree.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_TAGEXP_HEADER
#define KOLA_TAGEXP_HEADER 

#include "KGlobals.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "TYTObj.H"

#ifdef _OSTORE_
#include "OSObject.H"
#endif  // _OSTORE_

#include "PCState.H"
#include <iostream.h>

class TagExp : public NonType
{
public:
  TagExp ( TagExpName ten = TAGEXP_INVALID, char *n = NULL );
  // Constructor
  
  TagExp ( const TagExp & other);
  // Constructor
  
  virtual ~TagExp ();
  // Destructor
  
  virtual TagExp* Copy () const = 0;
  // Virtual Copy Constructor.

  NonType* Copy_kTree() const { return Copy(); }
  
  // TTagExp* Type() const{ return this->type; }
  // TObj* Type () const { return this->type; }
  // Selector.

  TagExpName Identity() const { return myname; }
  // what kind of tagexp are you?
  // return myname.
  
  void initCurrChild(){}
  void replaceChild(NonType* o, int n){}
  NonType* nextChild() { return NULL; } 
  
  virtual char *oql ();
  // Return the OQL string equivalent of this object
  
  virtual char *sql ();
  // Return the SQL string equivalent of this object
  
  virtual TagExp *tagexp();
  // Symbolic simplification of this.  Eventually,
  // this method will replace "object" presently
  // only defined with _OSTORE_ flag set
  
#ifdef _OSTORE_
  
  virtual TagExp *tagexp() = 0;
  // Returns the kola equivalent of an tagexp.

  virtual OSObject *result() = 0;
  // Returns the OSObject equivalent of itself.

  virtual int equal(TagExp* te) { return (this == te); }

  virtual int unequal(TagExp* te) { return (this != te); }

#endif // _OSTORE_
  
protected:
  // as we have not defined the TTagExp, this element is omitted
  // TObj* type;
  //  Objects always have some sort of Object Type.
  
  TagExpName  myname;
 // what kind of tagexp are you?
 
  /*********************************************************/
  // right now "myname" is not necessary. Myname() methods does not
  // use this data field. Myname() methods directly manipulates
  // BoolName for each classes.
  // 
  //  ObjName myname;
  //  What kind of Object are you?
  //*******************************************************/

  char * namestring;
  // Text representation for the name of the object.

};
#endif



