//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)					 	    
//  FILE: QLAuxCode.cc
//  PROJ: QL->KOLA Translator
//  DATE: 6/98
//  
//////////////////////////////////////////////////////////////////////////////

#include "QLAuxCode.H"

//////////////////////////////////////////////////////////////////////////////////
////////////////////////// Auxiliary Routines ////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
Predicate *CopyPred (Predicate *p) {

  // p
  if (p == NULL)
	  return NULL;
  else {
	  return p -> Copy ();
  }
}

Function *CopyFunc (Function *f) {

  // of
  if (f == NULL)
      return NULL;
  else {
      return f -> Copy ();
  }
}

Function *PairFunc (Function *f1, Function *f2) {

  if (f1 == NULL || f2 == NULL)
      return NULL;
  else {
      return new FPlus (f1 -> Copy (), f2 -> Copy ());
  }

}

Function *OptimizedCompose (Function *f1, Function *f2) {

  if (f1 == NULL || f2 == NULL)
    return NULL;
  
  else if (f1 -> Identity () == FUN_ID) {

    delete f1;
    return f2;

  } else if (f2 -> Identity () == FUN_ID) {

    delete f2;
    return f1;

  } else return new FCompose (f1, f2);

}
//////////////////////////////////////////////////////////////////////////////////
Predicate *OptimizedCombine (Predicate *p, Function *f) {

  if (p == NULL || f == NULL)
    return NULL;
  
  else if (f -> Identity () == FUN_ID) {

    delete f;
    return p;

  } else return new PCombine (p, f);

}


//////////////////////////// Rule Routines ///////////////////////////////////////
Environment *initenv () {
  Environment *nenv;
  nenv = new Environment();
  return nenv -> add_entry ("NULL", NULL);

}

// ============================== Q -> INTCON ==============================

Function *IntFunc (int i) {

 // K (i) 
 Function *res = new FConst (new OInt (i));
 return res;
}


// ============================== Q -> STRINGCON ==============================

Function *StrFunc (char *s) {

 // K (s) 
 return new FConst (new OStr (s));
}


// ============================== Q -> FLOATCON ==============================

Function *FloatFunc (char *s) {

 // K (s) 
 return new FConst (new OFloat (s));
}


// ============================== Q -> NULL =================================

Function *NilFunc () {

 // K (NULL)
 return new FConst (new OAny ());
}


// ============================== Q -> ELT (Q) ================================

Function *ElementFunc (Function *f) {

  if (f == NULL)
      return NULL;
  else
      //  SINGLETON o f2
      return OptimizedCompose (new FSingleton (), f -> Copy ());
}  

// ============================== Q -> ABS (Q) ================================

Function *AbsFunc (Function *f) {

  if (f == NULL)
      return NULL;
  else
      //  ABS o f2
      return OptimizedCompose (new FAbs (), f -> Copy ());
}  

// ============================== Q -> Q + Q ==============================

Function *PlusFunc (Function *f1, Function *f2) {

  if (f1 == NULL || f2 == NULL)
      return NULL;
  else {
	// ADD o <f1, f2>
	return new FCompose (new FAdd (), new FPlus (f1 -> Copy (), f2 -> Copy ()));
  }
}

// ============================== Q -> Q - Q ==============================

Function *MinusFunc (Function *f1, Function *f2) {

  if (f1 == NULL || f2 == NULL)
      return NULL;
  else
	// SUB o <f1, f2>
	return new FCompose (new FMinus (), new FPlus (f1 -> Copy (), f2 -> Copy ()));
}


// ============================== Q -> Q * Q ==============================

Function *MultFunc (Function *f1, Function *f2) {

  if (f1 == NULL || f2 == NULL)
      return NULL;
  else 
	// MUL o <f1, f2>
	return new FCompose (new FMult (), new FPlus (f1 -> Copy (), f2 -> Copy ()));
}


// ============================== Q -> Q / Q ==============================

Function *DivFunc (Function *f1, Function *f2) {

  if (f1 == NULL || f2 == NULL)
      return NULL;
  else
	// DIV o <f1, f2>
	return new FCompose (new FDiv (), new FPlus (f1 -> Copy (), f2 -> Copy ()));
}

// ============================== Q -> - Q  ==============================

Function *NegFunc (Function *f) {

  if (f == NULL)
      return NULL;
  else
	// C (MULT, -1) o f
	return new FCompose (new FCElt (new FMult (), new OInt (-1)), f -> Copy ());
}

// ============================== Q -> Q MOD Q ==============================

Function *ModFunc (Function *f1, Function *f2) {

  if (f1 == NULL || f2 == NULL)
      return NULL;
  else
  // MOD o <f1, f2>
  return new FCompose (new FMod (), new FPlus (f1 -> Copy (), f2 -> Copy ()));
}

// ============================== Q -> Q '=' Q ==============================

Predicate *EqPred (Function *f1, Function *f2) {

  if (f1 == NULL || f2 == NULL)
	  return NULL;
  else
	  // EQ OPLUS <f1, f2>
	  return new PCombine (new PEqual (), new FPlus (f1 -> Copy (), f2 -> Copy ()));
}

// ============================== Q -> Q '!=' Q ==============================

Predicate *NeqPred (Function *f1, Function *f2) {

  if (f1 == NULL || f2 == NULL)
	  return NULL;
  else
	  // NEQ OPLUS <f1, f2>
	  return new PCombine (new PNotEqual (), new FPlus (f1 -> Copy (), f2 -> Copy ()));
}

// ============================== Q -> Q '<' Q ==============================

Predicate *LTPred (Function *f1, Function *f2) {

  if (f1 == NULL || f2 == NULL)
	  return NULL;
  else
	  // LT OPLUS <f1, f2>
	  return new PCombine (new PLess (), new FPlus (f1 -> Copy (), f2 -> Copy ()));
}

// ============================== Q -> Q '>' Q ==============================

Predicate *GTPred (Function *f1, Function *f2) {

  if (f1 == NULL || f2 == NULL)
	  return NULL;
  else
	  // GT OPLUS <f1, f2>
	  return new PCombine (new PGreater (), new FPlus (f1 -> Copy (), f2 -> Copy ()));
}

// ============================== Q -> Q '<=' Q ==============================

Predicate *LEQPred (Function *f1, Function *f2) {

  if (f1 == NULL || f2 == NULL)
	  return NULL;
  else
	  // LEQ OPLUS <f1, f2>
	  return new PCombine (new PLessEqual (), new FPlus (f1 -> Copy (), f2 -> Copy ()));
}

// ============================== Q -> Q '>=' Q ==============================

Predicate *GEQPred (Function *f1, Function *f2) {

  if (f1 == NULL || f2 == NULL)
	  return NULL;
  else
	  // GEQ OPLUS <f1, f2>
	  return new PCombine (new PGreaterEqual (), new FPlus (f1 -> Copy (), f2 -> Copy ()));
}

// ============================== Q -> Q IS NULL ==============================

Predicate *IsNullPred (Function *f) {

  if (f == NULL)
	  return NULL;
  else
	  // ISNULL OPLUS f
	  return new PCombine (new PIsnull (), f -> Copy ());
}


// ============================== Q -> Q IS NULL ==============================

Predicate *IsNotNullPred (Function *f) {

  if (f == NULL)
	  return NULL;
  else
	  // C (EQ, NULL) OPLUS f
	  return new PCombine (new PIsnotnull (), f -> Copy ());
}


// ============================== Q -> NOT Q ==============================

Predicate *NotPred (Predicate *p) {

  // ~ (p)
  if (p == NULL)
      return NULL;
  else {
      return new PNegate (p -> Copy ());
  }
}

// ============================== Q -> Q AND Q ==============================

Predicate *AndPred (Predicate *p1, Predicate *p2) {

  // p1 & p2
  if (p1 == NULL || p2 == NULL)
      return NULL;
  else {
      return new PConjunct (p1 -> Copy (), p2 -> Copy ());
  }
}


// ============================== Q -> Q OR Q ==============================

Predicate *OrPred (Predicate *p1, Predicate *p2) {

  // p1 | p2
  if (p1 == NULL || p2 == NULL)
      return NULL;
  else {
      return new PDisjunct (p1 -> Copy (), p2 -> Copy ());
  }
}


// ============================== Q -> FLATTEN (Q) ==============================

Function *FlattenFunc (Function *f) {
  //  flat o f
  if (f == NULL)
	  return NULL;
  else
	  return OptimizedCompose (new FFlatten (), f -> Copy ());
}


// ============================== Q -> Q UNION Q ==============================

Function *UniFunc (Function *f1, Function *f2) {

  // UNI (=) o <f1, f2>
  if (f1 == NULL || f2 == NULL)
	  return NULL;
  else
	  return new FCompose (new FUni (new PEqual ()), new FPlus (f1 -> Copy (), f2 -> Copy ()));
}


// ============================== Q -> Q INTERSECT Q ==============================

Function *InterFunc (Function *f1, Function *f2) {

  // INT (=) o <f1, f2>
  if (f1 == NULL || f2 == NULL)
	  return NULL;
  else
	  return new FCompose (new FInt (new PEqual ()), new FPlus (f1 -> Copy (), f2 -> Copy ()));
}


// ============================== Q -> Q EXCEPT Q ==============================

Function *DifFunc (Function *f1, Function *f2) {

  // DIF (=) o <f1, f2>
  if (f1 == NULL || f2 == NULL)
	  return NULL;
  else
	  return new FCompose (new FDif (new PEqual ()), new FPlus (f1 -> Copy (), f2 -> Copy ()));
}

// ============================== Q -> Q IN Q ==============================

Predicate *InPred (Function *f1, Function *f2) {

  // EX (=) OPLUS <f1, f2>
  if (f1 == NULL || f2 == NULL)
	  return NULL;
  else
	  return new PCombine (new PCEX (new PEqual ()), new FPlus (f1 -> Copy (), f2 -> Copy ()));
}

// ============================== Q -> COUNT (Q) ===========================

Function *COUNTFunc (Function *f) {

  // FOLD (0, K (1), ADD) o f
  if (f == NULL)
	  return NULL;
  else
	  return OptimizedCompose (new FCount (), f -> Copy ());

}


// ============================== Q -> SUM (Q) ===========================

Function *SUMFunc (Function *f) {

  // FOLD (0, ID, ADD) o f
  if (f == NULL)
	  return NULL;
  else
	  return OptimizedCompose (new FSum (), f -> Copy ());

}

// ============================== Q -> MIN (Q) ===========================

Function *MINFunc (Function *f) {

  // PFOLD (id, con (lt, fst, snd))
  if (f == NULL)
	  return NULL;
  else
	  return OptimizedCompose (new FMin (), f -> Copy ());
}

// ============================== Q -> MAX (Q) ===========================

Function *MAXFunc (Function *f) {

  // PFOLD (id, con (lt, snd, fst))
  if (f == NULL)
	  return NULL;
  else
	  return OptimizedCompose (new FMax (), f -> Copy ());
}

// ============================== Q -> AVG (Q) ===========================

Function *AVGFunc (Function *f) {

  // PCON (C (<>, 0), FOLD ([0, 0], <id, K (1)>, ADD x ADD)) o f
  if (f == NULL)
	  return NULL;
  else
	  return OptimizedCompose (new FAvg (), f -> Copy ());
}

// ============================== Q -> FORALL IDENT IN Q : (Q) ===========================

Environment *QuanEnv (Environment *e, char *s) {
  Environment *nenv;
  nenv = new Environment(*e);

  if (e == NULL || s == NULL)
      return NULL;
  else	
    return nenv -> add_entry (s, NULL);
}


Predicate *FORALLPred (Function *f, Predicate *p) {

  // FA (p) OPLUS <id, f>
  if (f == NULL || p == NULL)
	  return NULL;
  else
	  return new PCombine (new PCFA (p -> Copy ()), new FPlus (new FID (), f -> Copy ()));
}

// ============================== Q -> EXISTS IDENT IN Q : (Q) ===========================

Predicate *EXISTSPred (Function *f, Predicate *p) {

  // EX (p) OPLUS <id, f>
  if (f == NULL || p == NULL)
	  return NULL;
  else
	  return new PCombine (new PCEX (p -> Copy ()), new FPlus (new FID (), f -> Copy ()));
}

// ============================== WhereOpt -> Q  ===========================

// ============================== Q -> SELECT Q_F FROM Q1, ..., Qn WhereOpt ======================
//
// As a whole, function generated should be:
//
//   iterate (WhereOpt, Q_F) o 
//    iterate (K (true), flat o iter (K (true), id) o <id, Q_n>) o 
//     ...
//      iterate (K (true), flat o iter (K (true), id) o <id, Q_1>) o 
//       singleton
//
// To generate, have Q1, ..., Qn (QEnv) generate flat o iterate (K (true), iter (K (true), id) o <id, T (Q_i)>) (1<=i<=n)
//  and rest generated by Q

Function *SFWFunc (Function *of, Function *f1, Function *f2, Predicate *p, int d) {

  // sort o iterate (p, f1) o f2 o set
  if (of == NULL || f1 == NULL || f2 == NULL || p == NULL)
      return NULL;
  else {
	  FCompose *f = new FCompose (new FIterate (p -> Copy (), f1 -> Copy ()),
								  new FCompose (f2 -> Copy (), new FSingleton ()));

	  if (d == 1) 
		return OptimizedCompose (of -> Copy (), new FCompose (new FSet (), f)); 
	  else 
		return OptimizedCompose (of -> Copy (), f);
  }
}



// ============================== QEnv -> Q IDENT  ===========================

Environment *QEnvEnv (Environment *env, char *s) {

  // Adds (s, t) to env and returns result
  if (env == NULL || s == NULL)
      return NULL;
  else {
	Environment *res = new Environment (*env);
	res -> add_entry (s, NULL);
	return res;
  }
}
	
Function *QEnvFunc (Function *f) {

  // flat o iterate (K (t), iter (K (t), id) o <id, f>)
  // New:  unnest (id, f) ! A =
  //          {[a, z] | a in A, z in (f ! a)}
  //
  //       flat ! (iterate (K (t), iter (K (t), id) o <id, f>) ! A)
  //        = flat ! {iter (K (t), id) ! [a, f ! a] | a in A}
  //        = flat ! {{[a, z] | z in (f ! a)} | a in A}
  //        = {[a, z] | a in A, z in (f ! a)}

  if (f == NULL)
	  return NULL;
  else
      return new FUnnest (new FID (), f -> Copy ());

	//	  return new FCompose (new FFlatten (), 
	//						   new FIterate (new PConst (new BPrimitive (T)), 
	//										 new FCompose (new FIter (new PConst (new BPrimitive (T)), 
	//																  new FID ()), 
	//													   new FPlus (new FID (), f -> Copy ()))));
}

// ============================== QEnv -> QEnv, Q IDENT  ===========================

Function *QEnvFunc2 (Function *f1, Function *f2) {

  // flat o iterate (K (t), iter (K (t), id) o <id, f2>) o f1
  
  if (f1 == NULL || f2 == NULL)
      return NULL;
  else 
      return new FCompose (new FUnnest (new FID (), f2 -> Copy ()), f1 -> Copy ());
	//      return new FCompose (new FCompose (new FFlatten (), 
	//										 new FIterate (new PConst (new BPrimitive (T)), 
	//													   new FCompose (new FIter (new PConst (new BPrimitive (T)), 
	//																				new FID ()), 
	//																	 new FPlus (new FID (), f2 -> Copy ())))), f1 -> Copy ());
  
}
// ============================== DPrL -> IDENT ':' E ==============================

// ============================== DPrL -> DPrL , IDENT : E ==============================

// ============================== Q -> IDENT ==============================

Function *IdentFunc (char *s, Environment *e) {

  Function *res;

  if (e == NULL || s == NULL)
      return NULL;
  else {
      res = e -> GetFunc (s);
      if (res != NULL) {
		 return res -> Copy ();
      } else {
		 res = new FConst (new OName (s));
		 return res;
      }
  }
}

Predicate *IdentPred (char *s, Environment *e) {

  // Assumption here is that one cannot have a set of Bools.
  // (Although one can have a name that refers to an arbitrary Bool.)
  // Therefore, any identifier that names a Bool must be Global
  // and not a lambda -declared variable
    
  if (e == NULL || s == NULL)
      return NULL;
  else if (e -> isEmpty ())
      return NULL;
  else
      return new PConst (new BName (s));
}


// ============================== Q -> PExp  ==================================

// ============================== PExp -> Q '.' IDENT '(' QList ')  ===============

Function *Path1Func (char *s, Function *f, Function *af) {

  if (s == NULL || f == NULL || af == NULL)
      return NULL;
  else {
      return OptimizedCompose (new FUser (s, new TFunc (new TAny(), new TAny())), new FPlus (f -> Copy (), af -> Copy ()));
  }

}

Predicate *Path1Pred (char *s, Function *f, Function *af) {

  if (s == NULL || f == NULL || af == NULL)
      return NULL;
  else {
      return OptimizedCombine (new PUser (s, new TPred (new TAny ())), new FPlus (f -> Copy (), af -> Copy ()));
  }

}

// ============================== Q -> Q '.' IDENT '(' ')  ===============

Function *Path2Func (char *s, Function *f) {

  if (s == NULL || f == NULL)
      return NULL;
  else {
      return OptimizedCompose (new FUser (s, new TFunc (new TAny (), new TAny ())), f -> Copy ());
  }

}

Predicate *Path2Pred (char *s, Function *f) {

  if (s == NULL || f == NULL)
      return NULL;
  else {
      return OptimizedCombine (new PUser (s, new TPred (new TAny ())), f -> Copy ());
  }

}

// ============================== QList -> Q  =========================
// ============================== QList -> QList ',' Q  =========================


// ============================== OrderOpt -> ORDER BY PExpList Direcn =====================
Function *Sort1Func (Function *f, Predicate *p) {

  if (f == NULL || p == NULL)
	return NULL;
  else 
	return new FSort (new PCombine (p -> Copy (), new FTimes (f -> Copy (), f -> Copy ())));
}

Function *Sort2Func (char *s, Predicate *p) {

  // ASC:  lt OPLUS <K (0), s>
  // DESC: gt OPLUS <K (0), s>

  //   s [x, y] = -1  (if x < y)
  //            = 0   (if x = y)
  //            = 1   (if x > y)

  if (s == NULL || p == NULL)
	return NULL;
  else 
	return new FSort (new PCombine (p -> Copy (), new FPlus (new FConst (new OInt (0)), new FUser (s, new TFunc (new TAny (), new TAny ())))));

}

