/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)					 	    
//  FILE: TYTFunc.H 
//  PROJ: KOLA
//  DATE: 5/95		                                                    
//
// SYNOPSIS
//  Concrete type.  This is the type
//  of function types, which are always formed with two
//  object types representing the domain and range
//  of the function.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_TFUNC_HEADER
#define KOLA_TFUNC_HEADER 

#include "../KGlobals.H"
#include "TYTObj.H"
#include "TYType.H"
#include <string.h>

class TFunc : public koType
{
    
 public: 
 TFunc (TObj* o1, TObj* o2);
  //  Constructor.  Takes domain and range types
  //  as arguments.
  
 TFunc (TFunc& other);
  //  Copy Constructor.
  
 TObj* Dom () {return one;}
  //  The domain type of the function type.
  
 TObj* Ran () {return two;}
  //  The range type of the function type.
  
  char* representation ();
  //  Return string representation of type.
  //  
  //    one -> representation () + "-->" + two -> representation ()

  TObj *Copy ();
  // Virtual Copy Constructor

  koType* transformation () {return Copy();}    


  char* prolog ();
  // Return string representation of type in prolog term.

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
  // convert kola tree into prolog term and return it.
#endif // _PROLOG_

  ~TFunc ();
    // Destructor.
    
 protected:
  TObj* one;
  //  The domain type of the function rtype.
  
  TObj* two;
  //  The range type of the function type.
  
};

#endif


